/*
  Show - VGA Planets Viewer
  (C) 2000 by Piotr Winiarczyk

  Version 0.8

  7.12.2000
  Bugs corected:
  -Temperature shown OK
  -Kore data contacts
  -Explore map support

  Planet info aviable (UNOWNED, FULL, SUPER SPY, EXPLORATION, TARGET, DARK SENSE)

  8.12.2000
  Planets colors

  9.12.2000
  Sensor sweep.
  Bioscan.
  Check if utilXX.dat is good.
  Max wrap when damage.

  18.12.2000
  Changes in definesions of ship status SHIPS_FULL, etc
  Mission.ini loading inproved
  Minefields are ploted (No sweep out info)

  21.12.2000
  Message reader-F3
  VCR Info -F4 -not completed

  22.12.2000
  Found bug in Left button pressed procedures
  Bases data read
  Version & Log

  23.12.2000
  Base display completed

  24.12.2000
  VCR info completed
  Improved Enemy ship view
  Minefield view inproved

  25.12.2000
  Universal FC scaned and displayed
  Bug corrected in ShowPlanet and ShowMinefield
  1024x768x32 800x600x32 640x480x32 video modes supported,
     use -3, -2, -1 (640x480x32) switches, or no switch for default video mode

   3. 1.2001
  Goto place on map while reading messages

   5. 1.2001
  Load explosions, got to them in mess reader
  Put heading in enemy ship screen

  25. 3.2001
  Clean up code, -Wall no warrings
  BUGFix: fclose(f) makes 'seg fault' when f==NULL on Linux

  Known bugs:
  Gravity well not considered in ETA
  Fuel burn formulas not exact

  TODO:
  READ WORMHOLES from util, kore
  SHIP BUILD SCREEN
  Read game info from util data
  Show ship projected trails + chunnel
  Help

*/


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <ctype.h>
#include "phostpdk.h"
#include "grx20.h"
#include "grxkeys.h"

#define  AND   &&
#define  OR    ||
#define  EQ    ==
#define  NEQ   !=
#define  GT    >
#define  GE    >=
#define  LT 	<
#define  LE    =<

#define  SHIPS_NOINFO  0
#define  SHIPS_PARTIAL 1
#define  SHIPS_FULL    2

#define  VIS_NORMAL    0
#define  VIS_OBJECTS   1
#define  VIS_PLANET    2
#define  VIS_SHIP      3
#define  VIS_BASE      4
#define  VIS_MINE      5

#define PLANETS_INFO_NO_INFO   0
#define PLANETS_INFO_FULL      1
#define PLANETS_INFO_SENSOR    2
#define PLANETS_INFO_DARK      4
#define PLANETS_INFO_EXPLORE   8
#define PLANETS_INFO_TARGET    16
#define PLANETS_INFO_SPY       32
#define PLANETS_INFO_BIOSCAN   64
#define PLANETS_INFO_UNOWNED   128

#define MINEFIELDS_INFO_NO_INFO    0
#define MINEFIELDS_INFO_FULL       1
#define MINEFIELDS_INFO_SWEPT_OUT  2


/*
sensor   - sensor sweep
spy      - super spy
unowned  - orbiting unowned planet - data in planetXX.dat but owner=0
*/

#define BASES_INFO_NO_INFO     0
#define BASES_INFO_FULL        1
#define BASES_INFO_EXIST       2
#define BASES_INFO_NO_BASE     4

#define   SS_NO_INFO           10
#define   SS_MINIMAL           0
#define   SS_LIGHT             1
#define   SS_MODERATE          2
#define   SS_SUBSTANCIAL       3
#define   SS_HEAVY             4
#define   SS_VERY_HEAVY        5

/* For objects found in starchart */
#define   MAX_OBJECTS          1000
#define   OBJECT_SHIP          1
#define   OBJECT_MINEFIELD     2
#define   OBJECT_PLANET        3
#define   OBJECT_NOT_FOUND     4
#define   OBJECT_MARKER        5

struct gendat_file{
              char   timestamp[18];
              Uns16  points[4][11];
              Uns16  race;
              char   passwd[20];
              char   notused;
              Uns32  shipsum,planetsum,basesum;
              Uns16  newpasswdflag;
              char   newpasswd[10];
              Uns16  turnnumber;
              Uns16  timestampsum;
                  };

struct more_ship_info {
                Uns16 heading;
                Uns16 mass;
                Uns16 infotype;
                Uns16 orbitingplanet;
                      };

struct shipxy_file {
                Uns16 X,Y,Owner,Mass;
                   };

struct target_file {
                Uns16 Id,Owner,Wrap,X,Y,Hull,Heading;
                char  Name[21];
                   };

struct record_head {
                     short recordtype;
                     short recordlen;
                   };

struct in_pos {
              Uns16 id,hull,owner,reowner,type;
              };

struct ExtMission_rec{
             char name[100];
             char tow[100];
             char inter[100];
             Boolean yestow;
             Boolean yesint;
             } ;

struct MessIndex {
               Uns32 Offset;
               Uns16 Length;
                 }  ;

struct VCRObject {
                 char Name[20];
                 Uns16 Damage,Crew,ID,Owner;
                 char Picture, Hull;
                 Uns16 BeamType,BeamNumber,NumBays,TorpType,NumTorp,NumTorpLuch;
                 };

struct VCRMain {
                Uns16 Init,Syg,Temp,BattleType,LMass,RMass;
                struct VCRObject LObject,RObject;
                Uns16 LShield,RShield;
               };


/* ************************************************************ */
/*                util.dat  structures                          */
/* ************************************************************ */

struct util_ally_rec
                   {
                char  OfferTo[11];
                char  OfferFrom[11];
                char  ConditionalTo[11];
                char  ConditionalFrom[11];
                   };

struct util_super_spy {
              Uns16 PlanetID;
              Uns16 Mines;
              Uns16 Factories;
              Uns16 Defences;
              char  FriendlyCode[4];  // only 3 chars are loaded, last is 0
              Uns32 Neutronium;
              Uns32 Tritanium;
              Uns32 Duranium;
              Uns32 Molybdenum;
              Uns32 Credits;
              Uns32 Supplies;
             };

struct util_dark_sense {
              Uns16 PlanetID;
              Uns16  Owner;
              Uns32 Neutronium;
              Uns32 Tritanium;
              Uns32 Duranium;
              Uns32 Molybdenum;
              Uns32 Credits;
              Uns16 HasStarbase;
             };

struct util_exploration {
              Uns16 PlanetID;
              Uns16 Temperature;
              Uns16 Owner;
              Uns32 Colonists;
              Uns16 HasStarbase;
             };

struct util_sensor_sweep {
              Uns16 PlanetID;
              Uns16 Owner;
              Uns16 Activity;
              };

struct util_base_target {
              Uns16 BaseID;
              Uns16 BaseOwner;
              };

struct util_planet_target {
              Uns16 PlanetID;
              Uns16 PlanetOwner;
              Uns16 Temperature;
              Uns16 NativesType;
              Uns16 NativeGovernment;
              Uns32 NumNatives;
              Uns32 Neutronium;
              Uns32 Tritanium;
              Uns32 Duranium;
              Uns32 Molybdenum;
              Uns32 NumColonists;
              Uns32 Supplies;
              Uns32 Credits;
              };


struct util_mine_scan {
              Uns16 MinefieldID;
              Uns16 XLocation;
              Uns16 YLocation;
              Uns16 Owner;
              Uns32 MinefieldUnits;
              Uns16 IsWeb;
              Uns16 FCodePlanet;
              Uns16 ScanType;
              };

struct util_wormhole_scan {
              Uns16 LocationX;
              Uns16 LocationY;
              Uns16 Mass;
              Uns16 StableCode;
              Uns16 ID;
              };
              
struct util_control {
              char  HostRunTime[18];
              Uns16 TurnNumber;
              Uns16 Player;
              char  PHOSTVersionMajor;
              char  PHOSTVersionMinor;
              Uns32 HullspecDigest;
              Uns32 EngspecDigest;
              Uns32 BeamspecDigest;
              Uns32 TorpspecDigest;
              Uns32 TruehullDigest;
              Uns32 PlanetXYDigest;
              Uns32 ConfigDigest;
              Uns32 RacenameDigest;
              char  GameName[32];
              char  PHOSTVersionRelease;
              };


struct util_bioscan {
              Uns16 PlanetID;
              Uns16 NativesType;
              Uns32 NativePopulation;
              Uns16 Temperature;
              };

struct util_battle_result {
              Uns16 LeftID, RightID;
              Uns16 IsPlanet;
              Uns16 LeftOwner,RightOwner;
              Uns16 LeftDamage,RightDamage;
              Uns16 LeftTorp,RightTorp;
              Uns16 LeftFighters,RightFighters;
              Uns16 LeftResult,RightResult;
              Uns16 X,Y;
                          };

struct util_explosion {
                      Uns16 X,Y,ID;
                      };
/* ******************* Main Global **************************** */

Uns16   race;
char    TimeSyg[20];
Uns16   turnnumber;
Uns16   explosion_number;

// event position
struct {
        Uns16    x,y,id,what;
        Boolean  go_to;
       } MessEvent;

/* ************************************************************ */

char    defaultpath[80]=".";
char    GraphicMode='0';

Uns16   ShipOnOrbit[501];
struct  more_ship_info LocalShips[501];
struct  in_pos objects_in_pos[MAX_OBJECTS];
Uns16   LocalPlanets[501];
Uns16   LocalBases[501];
Uns16   LocalMinefields[501];
GrColor PlanetColor[501];
GrColor MinefieldColor[501];
Uns16   MinefieldPlanetID[501];
char    SensorSweepActivity[501];
char    UniversalFC[12][4];
Uns16   UniversalFCPID[12];

struct MessIndex *MessageIndex;
char             *MessageData;
Uns16             MessageNumber;


struct util_explosion explosion[501];  // no of ships

struct  ExtMission_rec ExtMission[40];
Uns16   ExtMission_no=0;
Uns16   TubeMaxMass,BeamMaxMass;
Uns16   objects_in_pos_no;

/*
  VCR info read form util data,
  should be dynamical, but mor than 500 VCR is very imposible.
  VCRResultNumber is max index in this array
*/
struct  util_battle_result VCRBattleResult[500];
Uns16                      VCRBattleResultNumber;
// table to map vcr from vcrXX.dat to results from utilXX.dat
Uns16                      VCRBattleToBattleResult[500];

struct VCRMain  VCRBattle[500];
Uns16           VCRBattleNumber;

GrColor ColorBG;
GrColor ColorSt;
GrColor ColorSN;
GrColor ColorSI;
GrColor ColorSP;
GrColor ColorST;
GrColor ColorSRed;
GrColor ColorSGre;
GrColor ColorSYel;
// map
GrColor ColorSMRace;
GrColor ColorSMAlly;
GrColor ColorSMEnemy;

/*  Planets colors */
GrColor ColorMP;
GrColor ColorMS;
GrColor ColorPt;
GrColor ColorPN;
GrColor ColorPI;
GrColor ColorPRed;
GrColor ColorPWarning;

// map
GrColor ColorPMRace;
GrColor ColorPMAlly;
GrColor ColorPMEnemy;
GrColor ColorPMUnknown;

/* Minefields */
GrColor ColorMiRace;
GrColor ColorMiAlly;
GrColor ColorMiEnemy;

/* Color Messages */
GrColor ColorMessage;
GrColor ColorMessNow;

/* Base colors */
GrColor ColorBase;
GrColor ColorBaseStandard;
GrColor ColorBaseWarning;
GrColor ColorBaseDanger;

/* Procedure definitions */

void LoadShips(int);
Boolean ShowShip(int, int, int, int, int);
static char *ShipMissionName(Uns16);
Uns16 ShipTravelMass(Uns16);
Uns16 ShipTotalMass(Uns16);
Uns16 ShipCargoMass(Uns16);
static char *ShowInt4(int);
static char *ShowInt2(int);
static char *WhereItIs(int ,int );
int FindRemoteFromShipName(char *);
int FindShipTowee(Uns16 );
static char *ShipSpecialMission(int);
Boolean IsShipFCSpecial(const char *);
void PrepeareShowMap(void);
void ShowMap( int, int, int, int, int, int, int, int);
int MapY(int);
int MapX(int);
double  DistWrap(double, double, double, double);
double min(double, double);
double max(double, double);
void LoadAllPlayerData(void);
void LoadAllyFromUtil(int);
void  ShowObjectsAtPos(int, int, int, int, int);
void LoadMissionIniFile(void);
static char *ShipMissionExt(int);
void LoadPlanets(int);
static char *PlanetClimate(int);
static char *PlanetNatString(int);
static char *PlanetNatGovmString(int);
Boolean ShowPlanet(int, int, int, int, int);
int RND(double);
int ERND(double);
Uns16 PlanetMaxFactories(Uns16);
Uns16 PlanetMaxMines(Uns16);
Uns16 PlanetMaxDefense(Uns16);
Uns16 PlanetMining(Uns16, Uns16);
Uns16 PlanetGovTaxRate(Uns16 );
int PlanetNatHappyChange(Uns16);
int PlanetColHappyChange(Uns16);
Uns16 PlanetShipsHissing(Uns16);
Uns16 PlanetNatIncome(Uns16);
Uns16 PlanetColIncome(Uns16);
Uns16 PlanetSuppIncome(Uns16);
Boolean IsPlanetFCSpecial(const char *);
void IntegrateXYMaps(void);
void LoadUtilFile(int);
static char *SensorSweepString(char);
Boolean CheckUtilFile(int);
Boolean FindNearestMinefield(Uns16, Uns16, Uns16 *, Uns16 *);
Boolean FindNearestShip(Uns16, Uns16, Uns16 *, Uns16 *);
Boolean FindNearestPlanet(Uns16, Uns16,  Uns16 *, Uns16 *);
void BuildMinesAtPos(int ,int);
void BuildShipsAtPos(int ,int);
Boolean ShowMinefield(int ,int ,int, int, int);
void MessageReader(int, int, int, int);
void VCRReader(int, int, int, int);
void ShowVCRMess(int,int,struct VCRMain *, struct util_battle_result *);
void  LoadBases(int);
static char *BaseOrderString(int);
// static char *BaseFixString(int);
Boolean ShowBase(int, int, int, int, int);
Uns16 ShowBaseStorage(int, int, int, int, int, int);
static char *VCROutcomeString(int);
void ShowHullInfo(int, int, int, int, int, GrColor);
Boolean  LocateEventOnMap(char *);
int WrapYDist(int ,int );
int WrapXDist(int ,int );
int amin(int , int );


/* ------------------------------ */
double tmpx,tmpy;
double tmpxx,tmpyy;

/* version number */
Uns16 pMajor=0;
Uns16 pMinor=8;


/* Panels positions and fonts */

Uns16 LeftPanelX;
Uns16 LeftPanelDX;
Uns16 LeftPanelY;
Uns16 LeftPanelDY;

Uns16 RightPanelX;
Uns16 RightPanelDX;
Uns16 RightPanelY;
Uns16 RightPanelDY;

Uns16 ObjectListX;
Uns16 ObjectListDX;
Uns16 ObjectListY;
Uns16 ObjectListDY;

Uns16 FontDX;
Uns16 FontDY;

/* Other */

Boolean logF       = True;
char logname[80]   ="show.log";

static void
version(void)
{
  Info( "SHOW - VGA-Planets Viewer v%u.%u\n",pMajor, pMinor);
}

static void
usage(void)
{
	  Info(
"Usage: show [options] race [GameDirectory [RootDirectory]]\n"
"Options:\n"
"         v -- Print version and exit.\n"
"         h -- Print this help and exit.\n"
"         1 -- Use 640x480x32 video mode.\n"
"         2 -- Use 800x600x32 video mode.\n"
"         3 -- Use 1024x768x32 video mode.\n"
);
	  exit(1);
}

static void processOptions(int argc, char *argv[])
{
    int c=1;

    while ((c=pgetopt(argc, argv, "vh123")) NEQ EOF) {
		switch (c) {
		 case 'v':
			version();
			exit(1);
		 case 'h':
			usage();
		 case '1':
			GraphicMode='1';
			break;
		 case '2':
			GraphicMode='2';
			break;
		 case '3':
			GraphicMode='3';
			break;
                       }
           }

     c=poptind;

    race = atoi(argv[c++]);

    if ((race<1)||(race>11)) usage();

    if (c < argc) {
        gGameDirectory = argv[c++];
	}

    if (c < argc) {
        gRootDirectory = argv[c++];
	}

}

int  main(int argc, char *argv[])
{
     GrMouseEvent evt;

    
     Uns16 minpd,minsd,minmd;
     Uns16 minp,mins,minm;
     Boolean pf,sf,mf;
     Uns16 found;

     Uns16 storage_page;
     Uns32 wrapdx,wrapdy;

     Uns16 r,o;

     Uns32 c;
     int ii,i,sx,sy;
     int dx,dy,zoom=1;
     int mx,my;
     char buf[80];
     int vis_mode;
     int vis_object;

     Boolean mv,t;
     Uns16 planet_id;

  /* Fonts */

     FontDX = 8;
     FontDY = 12;

  /* Colors */

       ColorBG = GrAllocColor(0x00,0x00,0x00);
       ColorSt = GrAllocColor(0xff,0xff,0xff);
       ColorSN = GrAllocColor(0x80,0x80,0x80);
       ColorSI = GrAllocColor(0x00,0x80,0x80);
       ColorSP = GrAllocColor(0x00,0xc0,0x00);
       ColorST = GrAllocColor(0x00,0x00,0x80);
     ColorSRed = GrAllocColor(0xff,0x00,0x00);
     ColorSGre = GrAllocColor(0x00,0xff,0x00);
     ColorSYel = GrAllocColor(0xff,0xff,0x00);
       ColorMP = GrAllocColor(0xff,0xff,0xff);
       ColorMS = GrAllocColor(0x00,0xff,0x00);

   ColorSMRace = GrAllocColor(0x00,0xff,0x00);
   ColorSMAlly = GrAllocColor(0x80,0x80,0x80);
  ColorSMEnemy = GrAllocColor(0xff,0x00,0x00);
   ColorPMRace = GrAllocColor(0x00,0xff,0x00);
   ColorPMAlly = GrAllocColor(0x00,0x00,0xff);
  ColorPMEnemy = GrAllocColor(0xff,0x00,0x00);
ColorPMUnknown = GrAllocColor(0xff,0xff,0xff);

       ColorPt = GrAllocColor(0xff,0xff,0xff);
       ColorPN = GrAllocColor(0x80,0x80,0x80);
       ColorPI = GrAllocColor(0x00,0xff,0x00);
     ColorPRed = GrAllocColor(0xff,0x00,0x00);
 ColorPWarning = GrAllocColor(0xff,0xff,0x00);

   ColorMiRace = GrAllocColor(0x00,0xff,0x00);
   ColorMiAlly = GrAllocColor(0x00,0x00,0xff);
  ColorMiEnemy = GrAllocColor(0xff,0x00,0x00);

  ColorMessage = GrAllocColor(0xff,0xff,0xff);
  ColorMessNow = GrAllocColor(0x00,0x80,0x80);


 ColorBase           = GrAllocColor(0xff,0xff,0xff);
 ColorBaseStandard   = GrAllocColor(0x50,0x50,0xff);
 ColorBaseWarning    = GrAllocColor(0xff,0xff,0x50);
 ColorBaseDanger     = GrAllocColor(0xff,0x50,0x50);


     gRootDirectory = defaultpath;
     gGameDirectory = defaultpath;

     processOptions(argc, argv);

     InitPHOSTLib();

     if (logF==True)
     {
      gLogFile=OpenOutputFile(logname,GAME_DIR_ONLY|TEXT_MODE);
      if (gLogFile==NULL)
      {
         ErrorExit("Cannot open log file %s. Exiting.",logname);
      }
     }

     version();

/*   make xyplan.dat file form xyplanXX.dat files. Explmap support */
     IntegrateXYMaps();

     if (!ReadGlobalData()) ErrorExit("Cannot load Global Data.\n");

     InitializePlanets();
     InitializeShips();
     InitializeBases();
     InitializeMines();
     InitializeHostGen();

     for (i=1;i<12;i++) PutPlayerIsActive(i,True);

/*  load ally form util file */
     for (i=1;i<12;i++) LoadAllyFromUtil(i);

/*  load ships */
     LoadAllPlayerData();

/*  bring remote control back */

    for (i=1;i<=500;i++)
     if (IsShipExist(i))
      if ((r=FindRemoteFromShipName(ShipName(i,NULL)))!=0)
       {
        o=ShipOwner(i);
        PutShipOwner(i,r);
        t=AllowShipRemoteControl(i,r);
        t=BeginShipRemoteControl(i,o);
/* Can't Remote for Enemy ship when aliances are not set */
        if (!t)
        {
          AllyAddRequest(o,r);
          PutPlayerAllowsAlly(o,r,ALLY_SHIPS,ALLY_STATE_ON);
          AllyAddRequest(r,o);
          PutPlayerAllowsAlly(r,o,ALLY_SHIPS,ALLY_STATE_ON);
          t=BeginShipRemoteControl(i,o);
/*        if (!t) ErrorExit("Cannot restore ShipRemoteControl. Ship ID : %d",i);*/
        }
       }

     (void) OpenOutputFile("nextturn.hst", GAME_DIR_ONLY);

     if (!WriteHostData())  ErrorExit("Cannot save Host Data.\n");
     if (!ReadGlobalData()) ErrorExit("Cannot load Global Data.\n");
     if (!ReadHostData())   ErrorExit("Cannot load Host Data.\n");

     LoadMissionIniFile();

  /* Found most heavy equipment */

     TubeMaxMass=0;
     for (i=1;i<=TORP_NR;i++)
        if (TorpTubeMass(i)>TubeMaxMass) TubeMaxMass=TorpTubeMass(i);

     BeamMaxMass=0;
     for (i=1;i<=BEAM_NR;i++)
        if (BeamMass(i)>BeamMaxMass) BeamMaxMass=BeamMass(i);

  /*  Prepare wraparound   */

     wrapdx = gPconfigInfo->WraparoundRectangle[2]-gPconfigInfo->WraparoundRectangle[0];
     wrapdy = gPconfigInfo->WraparoundRectangle[3]-gPconfigInfo->WraparoundRectangle[1];

     sx = (gPconfigInfo->WraparoundRectangle[2]+gPconfigInfo->WraparoundRectangle[0])/2;
     sy = (gPconfigInfo->WraparoundRectangle[3]+gPconfigInfo->WraparoundRectangle[1])/2;

     dx=wrapdx;
     dy=wrapdy;

  /* Prepare screen */

     switch(GraphicMode)
     {
      case '1':
        GrSetMode(GR_width_height_color_graphics,640,480,32);
      break;

      case '2':
        GrSetMode(GR_width_height_color_graphics,800,600,32);
      break;

      case '3':
       GrSetMode(GR_width_height_color_graphics,1024,768,32);
      break;

      default:
        GrSetMode(GR_default_graphics);
      break;
     }
     LeftPanelX   =0;
     LeftPanelDX  =GrSizeX()-FontDX*20-1;
     LeftPanelY   =0;
     LeftPanelDY  =GrSizeY()-1;

     RightPanelX  =GrSizeX()-FontDX*20;
     RightPanelDX =FontDX*20;
     RightPanelY  =FontDY*3;
     RightPanelDY =FontDY*36;

     ObjectListX  =GrSizeX()-40*FontDX;
     ObjectListDX =20*FontDX;
     ObjectListY  =0;
     ObjectListDY =FontDY*40;

     GrClearScreen(ColorBG);

     PrepeareShowMap();
     ShowMap(LeftPanelX,LeftPanelY,LeftPanelDX,LeftPanelDY,sx,sy,dx,dy);
     if(GrMouseDetect()) {
	    GrMouseEventMode(1);
	    GrMouseInit();
	    GrMouseSetColors(GrAllocColor(255,0,0),GrBlack());
	    GrMouseDisplayCursor();
            GrMouseSetLimits(LeftPanelX,LeftPanelY,LeftPanelDX,LeftPanelDY);
     }

     c=0;
     ii=1;
     vis_mode=VIS_NORMAL;
     while(c!=GrKey_Escape)
     {
      GrMouseGetEventT(GR_M_EVENT,&evt,0L);

      /*  Show cursor position  */

      if (evt.flags & (GR_M_MOTION))
      {
         mx=MapX( floor(((double)sx+ (double)(evt.x - tmpx)*tmpxx)) );
         my=MapY( floor(((double)sy- (double)(evt.y - tmpy)*tmpyy)) );
         sprintf(buf,"(%d,%d)",mx,my);
         GrTextXY(RightPanelX,1,buf,ColorSI,ColorBG);
      }

      /*  Mouse map scroll */

      if (evt.flags & (GR_M_MOTION))
     {
         mv=False;
         if (evt.x==LeftPanelX)  { sx-=dx/40; mv=True; }
         if (evt.x==LeftPanelDX) { sx+=dx/40; mv=True; }
         if (evt.y==LeftPanelY)  { sy+=dy/40; mv=True; }
         if (evt.y==LeftPanelDY) { sy-=dy/40; mv=True; }
         if (mv)
         {
         if (sx<gPconfigInfo->WraparoundRectangle[0]) sx+=wrapdx;
         if (sx>gPconfigInfo->WraparoundRectangle[2]) sx-=wrapdx;
         if (sy<gPconfigInfo->WraparoundRectangle[1]) sy+=wrapdy;
         if (sy>gPconfigInfo->WraparoundRectangle[3]) sy-=wrapdy;
         vis_mode=VIS_NORMAL;
         found=OBJECT_NOT_FOUND;
         ShowMap(LeftPanelX,LeftPanelY,LeftPanelDX,LeftPanelDY,sx,sy,dy,dx);
         }
      }

      /*  Search for planet, ship if mouse button pressed */

      if (evt.flags & GR_M_LEFT_DOWN)
      {

      /* Try if ship from list is pointed  */

       if (vis_mode==VIS_OBJECTS)
       {
         if ((evt.x>=ObjectListX)&&(evt.x <=ObjectListX+ObjectListDX)&&
             (evt.y>=ObjectListY)&&(evt.y < FontDY*objects_in_pos_no) )
         {
          if (objects_in_pos_no!=0)
          {
           ii= objects_in_pos[(int)floor(evt.y/FontDY)].id;
           if (objects_in_pos[(int)floor(evt.y/FontDY)].type==OBJECT_SHIP)
           {
           ShowShip(RightPanelX,RightPanelY,RightPanelDX,RightPanelDY,ii);
           vis_object=VIS_SHIP;
           }
           if (objects_in_pos[(int)floor(evt.y/FontDY)].type==OBJECT_MINEFIELD)
           {
           ShowMinefield(RightPanelX,RightPanelY,RightPanelDX,RightPanelDY,ii);
           vis_object=VIS_MINE;
           }
           }
         }
         else
         {
         vis_mode=VIS_NORMAL;
         }
       }


       if (vis_mode==VIS_NORMAL)
       {
         mx=MapX( floor(((double)sx+ (double)(evt.x - tmpx)*tmpxx)) );
         my=MapY( floor(((double)sy- (double)(evt.y - tmpy)*tmpyy)) );

         pf=FindNearestPlanet   (mx,my,&minp,&minpd);
         sf=FindNearestShip     (mx,my,&mins,&minsd);
         mf=FindNearestMinefield(mx,my,&minm,&minmd);

         GrMouseEraseCursor();
         ShowMap(LeftPanelX,LeftPanelY,LeftPanelDX,LeftPanelDY,sx,sy,dy,dx);

        /* What is the nearest object */
         if ((minpd<=minsd)&&(minpd<=minmd)) { found=OBJECT_PLANET; }
         else
         if (minsd<=minmd) { found=OBJECT_SHIP;}
         else
         { found=OBJECT_MINEFIELD;}

       /* Build object list */
         if (found==OBJECT_PLANET)
         {
             objects_in_pos_no=0;
             if (minpd==minsd) BuildShipsAtPos(PlanetLocationX(minp),PlanetLocationY(minp));
             if (minpd==minmd) BuildMinesAtPos(PlanetLocationX(minp),PlanetLocationY(minp));
         }
         else
         if (found==OBJECT_SHIP)
         {
             objects_in_pos_no=0;
             BuildShipsAtPos(ShipLocationX(mins),ShipLocationY(mins));
             if (minsd==minmd) BuildMinesAtPos(ShipLocationX(mins),ShipLocationY(mins));
         }
         else
         if (found==OBJECT_MINEFIELD)
         {
             objects_in_pos_no=0;
             BuildMinesAtPos(MinefieldPositionX(minm),MinefieldPositionY(minm));
         }

       /* Display */
         if (found==OBJECT_PLANET)
         {
          planet_id=minp;
          ShowPlanet(RightPanelX,RightPanelY,RightPanelDX,RightPanelDY,minp);
          vis_object=VIS_PLANET;
          if (objects_in_pos_no!=0)
          {
           ShowObjectsAtPos(ObjectListX,ObjectListY,ObjectListDX,ObjectListDY,race);
           vis_mode=VIS_OBJECTS;
          }
          else
           vis_mode=VIS_NORMAL;
         }
         else // not planet
         {
          if (objects_in_pos_no!=0)
          {
           ShowObjectsAtPos(ObjectListX,ObjectListY,ObjectListDX,ObjectListDY,race);
           vis_mode=VIS_OBJECTS;
          }
          else
           vis_mode=VIS_NORMAL;

          ii=objects_in_pos[0].id;

          if (found==OBJECT_SHIP)
          {
           ShowShip(RightPanelX,RightPanelY,RightPanelDX,RightPanelDY,ii);
           vis_object=VIS_SHIP;
          }
          if (found==OBJECT_MINEFIELD)
          {
          ShowMinefield(RightPanelX,RightPanelY,RightPanelDX,RightPanelDY,ii);
          vis_object=VIS_MINE;
          }
          } // display

         GrMouseDisplayCursor();
       }  //vis=normal
      }  // left button pressed

      if (evt.flags & (GR_M_KEYPRESS))
      {
      c=evt.key;

      /* ZOOM */
      mv=False;
      if (c==9)
      {
         mx=MapX( floor(((double)sx+ (double)(evt.x - tmpx)*tmpxx)) );
         my=MapY( floor(((double)sy- (double)(evt.y - tmpy)*tmpyy)) );
         sx=mx;
         sy=my;
         zoom++;
         dx/=2;
         dy/=2;
        if (zoom==6) { dx=wrapdx; dy=wrapdy; zoom=1; }
        mv=True;
      }
      if (c==271)
      {
         mx=MapX( floor(((double)sx+ (double)(evt.x - tmpx)*tmpxx)) );
         my=MapY( floor(((double)sy- (double)(evt.y - tmpy)*tmpyy)) );
         sx=mx;
         sy=my;
         zoom--;
         dx*=2;
         dy*=2;
        if (zoom==1) { dx=wrapdx; dy=wrapdy;}
        if (zoom==0) { dx=wrapdx/32; dy=wrapdy/32; zoom=5; }
        mv=True;
      }

      if (mv)
      {
      if (sx<gPconfigInfo->WraparoundRectangle[0]) sx+=wrapdx;
      if (sx>gPconfigInfo->WraparoundRectangle[2]) sx-=wrapdx;
      if (sy<gPconfigInfo->WraparoundRectangle[1]) sy+=wrapdy;
      if (sy>gPconfigInfo->WraparoundRectangle[3]) sy-=wrapdy;
      vis_mode=VIS_NORMAL;
      found=OBJECT_NOT_FOUND;
      ShowMap(LeftPanelX,LeftPanelY,LeftPanelDX,LeftPanelDY,sx,sy,dy,dx);
      }
      /* StarBase */
       if (found==OBJECT_PLANET)
       {
         if (c==GrKey_b)
         {
             ShowBase(RightPanelX,RightPanelY,RightPanelDX,RightPanelDY,planet_id);
             storage_page=1;
             vis_object=VIS_BASE;
         }

         if (vis_object==VIS_BASE)
         {
         if (c==GrKey_Home)
         {
             storage_page--;
             storage_page=ShowBaseStorage(RightPanelX,RightPanelY+FontDY*16,RightPanelDX,FontDY*16,planet_id,storage_page);
         }
   
         if (c==GrKey_End)
         {
             storage_page++;
             storage_page=ShowBaseStorage(RightPanelX,RightPanelY+FontDY*16,RightPanelDX,FontDY*16,planet_id,storage_page);
         }
         }
        /* Planet */

        if (c==GrKey_p)
        {
            ShowPlanet(RightPanelX,RightPanelY,RightPanelDX,RightPanelDY,planet_id);
            vis_object=VIS_PLANET;
        }
      }

      /* Other keys */
      if (c==GrKey_F3)
      {
          MessageReader(LeftPanelX,LeftPanelY,LeftPanelDX,LeftPanelDY);
          vis_mode=VIS_NORMAL;

          if (MessEvent.go_to)
          {
          sx=MessEvent.x;
          sy=MessEvent.y;
          ShowMap(LeftPanelX,LeftPanelY,LeftPanelDX,LeftPanelDY,sx,sy,dy,dx);

     /* Almost same code as for left button search */
         pf=FindNearestPlanet   (sx,sy,&minp,&minpd);
         sf=FindNearestShip     (sx,sy,&mins,&minsd);
         mf=FindNearestMinefield(sx,sy,&minm,&minmd);

       /* Build object list */
         found =MessEvent.what;

         if (found==OBJECT_PLANET)
         {
             objects_in_pos_no=0;
             if (minpd==minsd) BuildShipsAtPos(PlanetLocationX(minp),PlanetLocationY(minp));
             if (minpd==minmd) BuildMinesAtPos(PlanetLocationX(minp),PlanetLocationY(minp));
         }
         else
         if (found==OBJECT_SHIP)
         {
             objects_in_pos_no=0;
             BuildShipsAtPos(ShipLocationX(mins),ShipLocationY(mins));
             if (minsd==minmd) BuildMinesAtPos(ShipLocationX(mins),ShipLocationY(mins));
         }
         else
         if (found==OBJECT_MINEFIELD)
         {
             objects_in_pos_no=0;
             BuildMinesAtPos(MinefieldPositionX(minm),MinefieldPositionY(minm));
         }

       /* Display */
         if (found==OBJECT_PLANET)
         {
          planet_id=MessEvent.id;
          ShowPlanet(RightPanelX,RightPanelY,RightPanelDX,RightPanelDY,MessEvent.id);
          vis_object=VIS_PLANET;
          if (objects_in_pos_no!=0)
          {
           ShowObjectsAtPos(ObjectListX,ObjectListY,ObjectListDX,ObjectListDY,race);
           vis_mode=VIS_OBJECTS;
          }
          else
           vis_mode=VIS_NORMAL;
         }
         else // not planet
         {
          if (objects_in_pos_no!=0)
          {
           ShowObjectsAtPos(ObjectListX,ObjectListY,ObjectListDX,ObjectListDY,race);
           vis_mode=VIS_OBJECTS;
          }
          else
           vis_mode=VIS_NORMAL;

          ii=MessEvent.id;

          if (found==OBJECT_SHIP)
          {
           ShowShip(RightPanelX,RightPanelY,RightPanelDX,RightPanelDY,ii);
           vis_object=VIS_SHIP;
          }
          if (found==OBJECT_MINEFIELD)
          {
          ShowMinefield(RightPanelX,RightPanelY,RightPanelDX,RightPanelDY,ii);
          vis_object=VIS_MINE;
          }
          } // display

          }
          else
          ShowMap(LeftPanelX,LeftPanelY,LeftPanelDX,LeftPanelDY,sx,sy,dy,dx);
      }
      if (c==GrKey_F4)
      {
          VCRReader(LeftPanelX,LeftPanelY,LeftPanelDX,LeftPanelDY);
          vis_mode=VIS_NORMAL;
          ShowMap(LeftPanelX,LeftPanelY,LeftPanelDX,LeftPanelDY,sx,sy,dy,dx);
      }

      }  // Key Pressed

     }
     MemFree(MessageIndex);
     MemFree(MessageData);
     GrMouseUnInit();
     GrSetMode(GR_default_text);
     if ((logF==True)&&(gLogFile!=NULL)) fclose(gLogFile);
     FreePHOSTLib();
     return 0;
}

/*
   Find ships in orbits of planets
   Color planets
   Color minefields
*/
void PrepeareShowMap(void)
{
  int i,j;
  GrColor C;
//  struct MineCircle *MC;

 // ships
  for (i=1;i<=500;i++) ShipOnOrbit[i]=0;
  for (i=1;i<=500;i++) LocalShips[i].orbitingplanet=0;

  for (i=1;i<=500;i++)
  {
    if (IsShipExist(i))
      for (j=1;j<=500;j++)
       if (IsPlanetExist(j))
        if ((PlanetLocationX(j)==ShipLocationX(i))&&(PlanetLocationY(j)==ShipLocationY(i)))
        {
          ShipOnOrbit[j]=i;
          LocalShips[i].orbitingplanet=j;
          break;
        }
  }

 // color planets
  for (i=1;i<=500;i++)
  {
    if (IsPlanetExist(i))
    {
       if (PlanetOwner(i)==race) C=ColorPMRace;
         else
       if (PlayerAllowsAlly(race,PlanetOwner(i),ALLY_PLANETS)) C=ColorPMAlly;
         else
       if ((LocalPlanets[i]>0)&&((LocalPlanets[i]&PLANETS_INFO_UNOWNED)==0))
       {
         if (( LocalBases[i]&BASES_INFO_EXIST)==BASES_INFO_EXIST)
           C=ColorPRed;
         else
           C=ColorPWarning;
       }
       else
        C=ColorPMUnknown;
       PlanetColor[i]=C;
    }
    else // planet not exist
    PlanetColor[i]= ColorBG;
  }

// color minefields
  for (i=1;i<=500;i++)
  {
    if (IsMinefieldExist(i))
    {
      if (MinefieldOwner(i)==race) C=ColorMiRace;
      else
      if (PlayerAllowsAlly(race,MinefieldOwner(i),ALLY_MINES)) C=ColorMiAlly;
      else
      C=ColorMiEnemy;

     MinefieldColor[i]=C;
    }
  }

}

void ShowMap( int sex,int sey,
              int dex,int dey,
              int sx ,int sy,
              int dx ,int dy)
{
 int i,pr,mine_ry,mine_rx;
 int wex,wey;
 int wexx,weyy;
 int WrapDX,WrapDY,X,Y;
 GrColor C= GrAllocColor(0x00,0x00,0x00);

 if (dx<gPconfigInfo->WraparoundRectangle[2]-gPconfigInfo->WraparoundRectangle[0])
 pr=1; else pr=0;

 tmpx  = sex+(double)(dex+1)/2.0;
 tmpxx = (double)(dx+0)/(double)(dex+1);

 tmpy  = sey+(double)(dey+1)/2.0;
 tmpyy = (double)(dy+0)/(double)(dey+1);

 GrSetClipBox(sex,sey,sex+dex,sey+dey);
 GrFilledBox(sex,sey,sex+dex,sey+dey,ColorBG);

// ships
 for(i=1;i<=500;i++)
 if (IsShipExist(i))
 {
   wex = tmpx + (WrapXDist(sx,ShipLocationX(i)) )/tmpxx;
   wey = tmpy + (WrapYDist(sy,ShipLocationY(i)) )/tmpyy;
   if (ShipOwner(i)==race) 
     C=ColorSMRace;
   else
   {
   if (PlayerAllowsAlly(race,ShipOwner(i),ALLY_SHIPS)) 
      C=ColorSMRace;
   else
      C=ColorSMEnemy;
   }  
   
   if (LocalShips[i].orbitingplanet)
   {
   if (C!=ColorSMEnemy)
        GrCircle(wex,wey,3,C);
   else 
        GrCircle(wex,wey,2,C);
   }
   else
       GrCircle(wex,wey,0,C);
 }

// planets
 for(i=1;i<=500;i++)
 if (IsPlanetExist(i))
 {
   wex = tmpx + (WrapXDist(sx,PlanetLocationX(i)) )/tmpxx;
   wey = tmpy + (WrapYDist(sy,PlanetLocationY(i)) )/tmpyy;

   C=PlanetColor[i];
   if (pr)
    {
    GrCircle(wex,wey,pr,C);
    GrPlot (wex,wey,C);
    }
   else
    GrPlot (wex,wey,C);
 }

// mines
WrapDX=gPconfigInfo->WraparoundRectangle[2]-gPconfigInfo->WraparoundRectangle[0];
WrapDY=gPconfigInfo->WraparoundRectangle[3]-gPconfigInfo->WraparoundRectangle[1];

for(i=1;i<=500;i++)
 if (IsMinefieldExist(i))
 {
   C=MinefieldColor[i];
   mine_rx = MinefieldRadius(i)/tmpxx;
   mine_ry = MinefieldRadius(i)/tmpyy;
   X = MinefieldPositionX(i);
   Y = MinefieldPositionY(i);

   wex = tmpx + (X-sx)/tmpxx;
   wey = tmpy - (Y-sy)/tmpyy;
   GrEllipse(wex,wey,mine_rx,mine_ry,C);

   if (X > sx) wexx = tmpx + (X-sx-WrapDX)/tmpxx;
   else        wexx = tmpx + (X-sx+WrapDX)/tmpxx;
   GrEllipse(wexx,wey,mine_rx,mine_ry,C);

   if (Y > sy) weyy = tmpy - (Y-sy-WrapDY)/tmpyy;
   else        weyy = tmpy - (Y-sy+WrapDY)/tmpyy;
   GrEllipse(wex,weyy,mine_rx,mine_ry,C);

   GrEllipse(wexx,weyy,mine_rx,mine_ry,C);

 }

  GrResetClipBox();
}

void LoadAllPlayerData(void)
{
   Uns16 init_tmp[11];    /* contens of init.tmp file */
   Uns32 fizz_bin[3][11]; /* contens of begining fizz.bin file */
   struct gendat_file gendat;
   Uns16 shipno,sID;
   char   signature1[10],signature2[10];
   FILE *f;
   int i,j,k;
   char s[80];
   char buf[400];  /* Temp buffer */
   Boolean readthis;
   Boolean VCRNotLoaded;
   struct shipxy_file buf_xy;
   struct target_file buf_target;
   Uns32 shipkore;
   struct MessIndex mtmp;
//   struct MessIndex *messtable;
   struct VCRMain VCRtmp;

   /* Read init.tmp*/
   f=OpenInputFile("init.tmp",GAME_DIR_ONLY);
   fread(&init_tmp,sizeof(init_tmp),1,f);
   if (f!=NULL) fclose(f);
   /* Read fizz.bin*/
   f=OpenInputFile("fizz.bin",GAME_DIR_ONLY);
   fread(&fizz_bin,sizeof(fizz_bin),1,f);
   if (f!=NULL) fclose(f);

   if (init_tmp[race-1]!=1)
     ErrorExit("Race %d data not found in game dir %s ",race,gGameDirectory);
   /* Load gen data for main player */
   sprintf(s,"gen%d.dat",race);
   f=OpenInputFile(s,GAME_DIR_ONLY);
   fread(&gendat,128,1,f);
   fread(&gendat.notused,1,1,f);
   fread(&gendat.shipsum,28,1,f);
   if (f!=NULL) fclose(f);

   strncpy(TimeSyg,gendat.timestamp,18);
   TimeSyg[18] = 0;
   turnnumber  = gendat.turnnumber;

   VCRBattleResultNumber =0;
   VCRBattleNumber       =0;
   /* Make all ships aviable */
   /* Other preparations */


   for(j=1;j<=500;j++)
   {
   CreateShip(15);
   LocalShips[j].mass=0;
   LocalShips[j].heading=0;
   LocalShips[j].infotype=SHIPS_NOINFO;
   LocalShips[j].orbitingplanet=0;
   LocalPlanets[j]=PLANETS_INFO_NO_INFO;
   LocalBases[j]  =BASES_INFO_NO_INFO;

   SensorSweepActivity[j] = SS_NO_INFO;

   CreateMinefield(100,100,15,1,False);
   MinefieldPlanetID[j]=0;
   LocalMinefields[j]=MINEFIELDS_INFO_NO_INFO;
   }

/*
   Load player messages.
   No signature check since there is no one in mdataXX.dat file.
*/
   sprintf(buf,"mdata%d.dat",race);
   f=OpenInputFile(buf,GAME_DIR_ONLY);
 
   fread(&MessageNumber,sizeof(MessageNumber),1,f);
   if (MessageNumber!=0)
   {
   MessageIndex= MemCalloc(MessageNumber,sizeof(mtmp));
   for (i=0;i<MessageNumber;i++)
     fread(MessageIndex+i,6,1,f);
 
   i=(MessageIndex+MessageNumber-1)->Offset-MessageIndex->Offset+(MessageIndex+MessageNumber-1)->Length;
   MessageData=MemAlloc(i);
   fread(MessageData,i,1,f);
 
   // decode messages
   for (j=0;j<i;j++)
      *(MessageData+j)=*(MessageData+j)-13;
   }
   if (f!=NULL) fclose(f);


   for (i=1;i<12;i++)
   if (init_tmp[i-1]==1) /* Player data must exist */
   {
      /* Read genXX.dat */
    sprintf(s,"gen%d.dat",i);
    f=OpenInputFile(s,GAME_DIR_ONLY);
    fread(&gendat,128,1,f);
    fread(&gendat.notused,1,1,f);
    fread(&gendat.shipsum,28,1,f);
     if (f!=NULL) fclose(f);
    Info("This is turn %d, race %d",gendat.turnnumber,gendat.race);
    if (gendat.turnnumber!=turnnumber)
    {
      Info("Not same turn number. Skipping player %d data.",i);
      break;
    }
    strncpy(buf,gendat.timestamp,18);
    buf[18]=0;
    if (strcmp(buf,TimeSyg)!=0)
    {
      Info("Not same time sygnatures. Skipping player %d data.",i);
      break;
    }
    /* Build signatures */
    for (j=0;j<10;j++)
     signature1[j]=gendat.passwd[10+j];
    for (j=0;j<10;j++)
     signature2[j]=gendat.passwd[10+j]+j+1;


    /* scan shipxy data for signature */
    sprintf(s,"shipxy%d.dat",i);
    f=OpenInputFile(s,GAME_DIR_ONLY);
    for (k=0;k<500;k++)
     fread(buf,8,1,f);
    fread(buf,10,1,f);
    if (f!=NULL) fclose(f);
    readthis=True;
    for (j=0;j<10;j++)
     if (buf[j]!=signature2[j]) readthis=False;
    if (readthis)
    {
    Info("Reading shipxy%d.dat",i);
    sprintf(s,"shipxy%d.dat",i);
    f=OpenInputFile(s,GAME_DIR_ONLY);
    for (k=0;k<500;k++)
    {
    fread(&buf_xy,8,1,f);
    if (buf_xy.Owner!=0)
    {
     if (LocalShips[k+1].infotype<=SHIPS_PARTIAL)
     {
      LocalShips[k+1].mass    =buf_xy.Mass;
      LocalShips[k+1].infotype=SHIPS_PARTIAL;
      PutShipOwner(k+1,buf_xy.Owner);
      PutShipLocationX(k+1,buf_xy.X);
      PutShipLocationY(k+1,buf_xy.Y);
     }
    }
    }
    if (f!=NULL) fclose(f);
    }
    else          Info("Skipping shipxy%d.dat",i);

    /* scan target data for signature */
    sprintf(s,"target%d.dat",i);
    f=OpenInputFile(s,GAME_DIR_ONLY);
    fread(&shipno,sizeof(shipno),1,f);
    for (k=0;k<shipno;k++)
     fread(buf,34,1,f);
    fread(buf,10,1,f);
     if (f!=NULL) fclose(f);
    readthis=True;
    for (j=0;j<10;j++)
     if (buf[j]!=signature2[j]) readthis=False;
    if (readthis)
    {
    Info("Reading target%d.dat",i);
    sprintf(s,"target%d.dat",i);
    f=OpenInputFile(s,GAME_DIR_ONLY);
    fread(&shipno,sizeof(shipno),1,f);
    for (k=0;k<shipno;k++)
    {
     fread(&buf_target,34,1,f);
     sID=buf_target.Id;
     buf_target.Name[20]=0;
     if (LocalShips[sID].infotype<=SHIPS_PARTIAL)
     {
      LocalShips[sID].infotype=SHIPS_PARTIAL;
      LocalShips[sID].heading =buf_target.Heading;
      PutShipOwner(sID,buf_target.Owner);
      PutShipSpeed(sID,buf_target.Wrap);
      PutShipLocationX(sID,buf_target.X);
      PutShipLocationY(sID,buf_target.Y);
      PutShipHull(sID,buf_target.Hull);
      PutShipName(sID,buf_target.Name);
     }
    }
     if (f!=NULL) fclose(f);
    }
    else Info("Skipping target%d.dat",i);

    /* scan target.ext data for signature */
    /* skip this file if there is no such */
    sprintf(s,"target%d.ext",i);
    f=OpenInputFile(s,GAME_DIR_ONLY|NO_MISSING_ERROR);
    if (f!=NULL)
    {
    fread(&shipno,sizeof(shipno),1,f);
    for (k=0;k<shipno;k++)
     fread(buf,34,1,f);
    fread(buf,10,1,f);
     if (f!=NULL) fclose(f);
    readthis=True;
    for (j=0;j<10;j++)
     if (buf[j]!=signature2[j]) readthis=False;
    if (readthis)
    {
    Info("Reading target%d.ext",i);
    sprintf(s,"target%d.ext",i);
    f=OpenInputFile(s,GAME_DIR_ONLY);
    fread(&shipno,sizeof(shipno),1,f);
    for (k=0;k<shipno;k++)
    {
     fread(&buf_target,34,1,f);
     sID=buf_target.Id;
     buf_target.Name[20]=0;
     if (LocalShips[sID].infotype<=SHIPS_PARTIAL)
     {
      LocalShips[sID].infotype=SHIPS_PARTIAL;
      LocalShips[sID].heading =buf_target.Heading;
      PutShipOwner(sID,buf_target.Owner);
      PutShipSpeed(sID,buf_target.Wrap);
      PutShipLocationX(sID,buf_target.X);
      PutShipLocationY(sID,buf_target.Y);
      PutShipHull(sID,buf_target.Hull);
      PutShipName(sID,buf_target.Name);
     }
    }
     if (f!=NULL) fclose(f);
    }
    else          Info("Skipping target%d.ext",i);
    }
    else
    Info("No target%d.ext",i);

/*  now try koreXX.dat for visual contacts */
    sprintf(s,"kore%d.dat",i);
    f=OpenInputFile(s,GAME_DIR_ONLY|NO_MISSING_ERROR);
    if (f!=NULL)
    {
/* check signature 2*/
    fread(buf,19,1,f);
     if (f!=NULL) fclose(f);
    readthis=True;
    for (j=0;j<10;j++)
     if (buf[j+9]!=signature2[j]) readthis=False;

    if (readthis)
    {
    Info("Reading kore%d.dat",i);
    sprintf(s,"kore%d.dat",i);
    f=OpenInputFile(s,GAME_DIR_ONLY);
/*  check if visual contacts are in koreXX.dat*/

    fseek(f,12702,SEEK_SET);
    fread(buf,4,1,f);
    if (buf[1]=='1') /*Yes load more*/
    {
    fread(buf,16,1,f);
    fread(&shipkore,sizeof(shipkore),1,f);
    for (k=0;k<shipkore;k++)
    {
     fread(&buf_target,34,1,f);
     sID=buf_target.Id;
/* decript name */
     for (j=0;j<20;j++) buf_target.Name[j]= (unsigned char)buf_target.Name[j] ^ (unsigned char)(155-(j+1));
     buf_target.Name[20]=0;
     if (LocalShips[sID].infotype<=SHIPS_PARTIAL)
     {
      LocalShips[sID].infotype=SHIPS_PARTIAL;
      LocalShips[sID].heading =buf_target.Heading;
      PutShipOwner(sID,buf_target.Owner);
      PutShipSpeed(sID,buf_target.Wrap);
      PutShipLocationX(sID,buf_target.X);
      PutShipLocationY(sID,buf_target.Y);
      PutShipHull(sID,buf_target.Hull);
      PutShipName(sID,buf_target.Name);
     }
    }   // ships
    } // 1120 sygnature
     if (f!=NULL) fclose(f);
    }
    else          Info("Skipping kore%d.dat",i);
    }
    else
    Info("No kore%d.dat",i);




    /* Scan ship data for signature */
    sprintf(s,"ship%d.dat",i);
    f=OpenInputFile(s,GAME_DIR_ONLY);
    fread(&shipno,sizeof(shipno),1,f);
    for (k=0;k<shipno;k++)
     fread(buf,107,1,f);
    fread(buf,10,1,f);
    if (f!=NULL) fclose(f);
    readthis=True;
    for (j=0;j<10;j++)
     if (buf[j]!=signature2[j]) readthis=False;
    if (readthis)
    {
    Info("Reading ship%d.dat",i);
    LoadShips(i);
    }
    else  Info("Skipping ship%d.dat",i);


    /* Scan planet data for signature */
    sprintf(s,"pdata%d.dat",i);
    f=OpenInputFile(s,GAME_DIR_ONLY);
    fread(&shipno,sizeof(shipno),1,f);
    for (k=0;k<shipno;k++)
     fread(buf,85,1,f);
    fread(buf,10,1,f);
    if (f!=NULL) fclose(f);
    readthis=True;
    for (j=0;j<10;j++)
     if (buf[j]!=signature2[j]) readthis=False;
    if (readthis)
    {
    Info("Reading pdata%d.dat",i);
    LoadPlanets(i);
    }
    else  Info("Skipping pdata%d.dat",i);

    /* Scan base data for signature */
    sprintf(s,"bdata%d.dat",i);
    f=OpenInputFile(s,GAME_DIR_ONLY);
    fread(&shipno,sizeof(shipno),1,f);
    for (k=0;k<shipno;k++)
    fread(buf,156,1,f);
    fread(buf,10,1,f);
     if (f!=NULL) fclose(f);
    readthis=True;
    for (j=0;j<10;j++)
     if (buf[j]!=signature2[j]) readthis=False;
    if (readthis)
    {
    Info("Reading bdata%d.dat",i);
    LoadBases(i);
    }
    else  Info("Skipping bdata%d.dat",i);


    /*   Try to load utilXX.dat  */
    if ( CheckUtilFile(i) )
    {
    Info ("Reading util%d.dat",i);
    LoadUtilFile(i);
    }
    else
    Info ("Skipping util%d.dat. File not from curent game.",i);


    sprintf(buf,"vcr%d.dat",i);
    f=OpenInputFile(buf,GAME_DIR_ONLY);
    fread(&shipno,sizeof(shipno),1,f);
    for (k=0;k<shipno;k++)
     fread(buf,100,1,f);
    fread(buf,10,1,f);
    if (f!=NULL) fclose(f);
    readthis=True;
    for (j=0;j<10;j++)
     if (buf[j]!=signature2[j]) readthis=False;
    if (readthis)
    {
    Info("Reading vcr%d.dat",i);
    sprintf(buf,"vcr%d.dat",i);
    f=OpenInputFile(buf,GAME_DIR_ONLY);
    fread(&shipno,sizeof(shipno),1,f);
 
    for (j=0;j<shipno;j++)
    {
      fread(&VCRtmp,100,1,f);
    // check if VCR is not allready loaded
      VCRNotLoaded=True;
      for (k=0;k<VCRBattleNumber;k++)
       if ((VCRtmp.LObject.ID==VCRBattle[k].LObject.ID)&&
           (VCRtmp.RObject.ID==VCRBattle[k].RObject.ID)&&
           (VCRtmp.BattleType==VCRBattle[k].BattleType))
               VCRNotLoaded=False;
      if (VCRNotLoaded)
      {
       // move data
        memmove(&VCRBattle[VCRBattleNumber],&VCRtmp,100);
        VCRBattleNumber++;
      }
     }// for each VCR in file
    if (f!=NULL) fclose(f);
    } //read this
    else  Info("Skipping vcr%d.dat",i);

   }  // player data

/* Clean up ships, mines */

   for(j=1;j<=500;j++)
   {
     if (ShipOwner(j)==15)      DeleteShip(j);
     if (MinefieldOwner(j)==15) PutMinefieldUnits(j,0);
   }

/* Scan for universal FCodes */
   for(j=1;j<=11;j++)
   {
     strcpy(UniversalFC[j],"");
     UniversalFCPID[j]=0;
   }

   for(j=1;j<=11;j++)
    for(i=1;i<=500;i++)
     {
      if ((LocalPlanets[i]&PLANETS_INFO_FULL)    ==PLANETS_INFO_FULL )
       {
         PlanetFCode(i,buf);
         buf[2]=0;
         if ((strcmp(buf,"mf")==0)&&(j==PlanetOwner(i)))
         {
            PlanetFCode(i,buf);
            strcpy(UniversalFC[j],buf);
            UniversalFC[j][3]=0;
            UniversalFCPID[j]=i;
         }
       } // no full info
    }

  // find battle results to VCRs
  for (j=0;j<VCRBattleResultNumber;j++)
    for (i=0;i<VCRBattleNumber;i++)
  if ((VCRBattleResult[j].LeftID  ==VCRBattle[i].LObject.ID)&&
      (VCRBattleResult[j].RightID ==VCRBattle[i].RObject.ID)&&
      (VCRBattleResult[j].IsPlanet==VCRBattle[i].BattleType))
        VCRBattleToBattleResult[i]=j;
}

void LoadShips(int tmprace)
{

   char name[20],buf_s[50];
   FILE *f;
   Uns16 howmany,i;
   Uns16 id;
   Uns16 buf;
   Uns16 x,y;


   sprintf(name,"ship%d.dat",tmprace);
   f=OpenInputFile(name,GAME_DIR_ONLY);

   fread(&howmany,sizeof(howmany),1,f);
   for(i=0;i<howmany;i++)
   {
     fread(&id,sizeof(id),1,f);
     fread(&buf,sizeof(buf),1,f);
     PutShipOwner(id,buf);
//     ShipOwnerLocal[id]=buf;
      LocalShips[id].infotype=SHIPS_FULL;

     fread(buf_s,3,1,f);
     buf_s[3]=0;
     PutShipFC(id,buf_s);
     fread(&buf,sizeof(buf),1,f);
     PutShipSpeed(id,buf);
     fread(&buf,sizeof(buf),1,f);
     x=buf;
     fread(&buf,sizeof(buf),1,f);
     y=buf;
     fread(&buf,sizeof(buf),1,f);
     PutShipLocationX(id,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipLocationY(id,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipRelWaypointX(id,x);
     PutShipRelWaypointY(id,y);
     PutShipEngine(id,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipHull(id,buf);
     fread(&buf,sizeof(buf),1,f);
     if (buf>0)
     PutShipBeamType(id,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipBeamNumber(id,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipBays(id,buf);
     fread(&buf,sizeof(buf),1,f);
     if (buf>0)
     PutShipTorpType(id,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipAmmunition(id,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipTubeNumber(id,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipMission(id,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipEnemy(id,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipTowTarget(id,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipDamage(id,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipCrew(id,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipCargo(id,COLONISTS,buf);
     fread(buf_s,20,1,f);
     buf_s[20]=0;
     PutShipName(id,buf_s);
     fread(&buf,sizeof(buf),1,f);
     PutShipCargo(id,NEUTRONIUM,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipCargo(id,TRITANIUM,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipCargo(id,DURANIUM,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipCargo(id,MOLYBDENUM,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipCargo(id,SUPPLIES,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipDump(id,NEUTRONIUM,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipDump(id,TRITANIUM,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipDump(id,DURANIUM,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipDump(id,MOLYBDENUM,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipDump(id,COLONISTS,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipDump(id,SUPPLIES,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipDumpPlanet(id,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipTransfer(id,NEUTRONIUM,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipTransfer(id,TRITANIUM,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipTransfer(id,DURANIUM,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipTransfer(id,MOLYBDENUM,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipTransfer(id,COLONISTS,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipTransfer(id,SUPPLIES,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipTransferShip(id,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipInterceptTarget(id,buf);
     fread(&buf,sizeof(buf),1,f);
     PutShipCargo(id,CREDITS,buf);
   }

   if (f!=NULL) fclose(f);
}


Boolean ShowShip(int x,int y,int ddx, int ddy, int id)
{

  char buf[100];
  char buf1[100];
  int dx=FontDX,dy=FontDY,iy=0,i;
  Uns16 ETA,mass,burn,repair,mwarp;
  int maxcargo,mincargo;
  double dis,fc;
  Boolean ShipWillHYP;


  if (!IsShipExist(id)) return False;

  GrSetClipBox(x,y,x+ddx,y+ddy);
  GrFilledBox(x,y,x+ddx,y+ddy,ColorBG);

  sprintf(buf,"Ship : %d",id);
  GrTextXY(x,y+dy*iy,buf,ColorSt,ColorBG);
  iy++;

  if (IsShipRemoteControlled(id))
   sprintf(buf,"Race : %s",RaceNameAdjective(ShipRemoteController(id),NULL));
  else
   sprintf(buf,"Race : %s",RaceNameAdjective(ShipOwner(id),NULL));
  GrTextXY(x,y+dy*iy,buf,ColorSt,ColorBG);
  iy++;
  if (IsShipRemoteControlled(id))
  {
    sprintf(buf," RCO : %s",RaceNameAdjective(ShipRemoteOwner(id),NULL));
    GrTextXY(x,y+dy*iy,buf,ColorSGre,ColorBG);
  }
  iy++;
  sprintf(buf,"%s",HullName(ShipHull(id),NULL));
  GrTextXY(x,y+dy*iy,buf,ColorSt,ColorBG);
  iy++;
  sprintf(buf,"%s",ShipName(id,NULL));
  GrTextXY(x,y+dy*iy,buf,ColorSN,ColorBG);
  iy++;
  iy++;
  sprintf(buf,"at %s",WhereItIs(ShipLocationX(id),ShipLocationY(id)));
  GrTextXY(x,y+dy*iy,buf,ColorSI,ColorBG);
  iy++;
  iy++;

  if ( (i=FindShipTowee(id)) !=0 )
  {
   sprintf(buf,"Being towed by %d",i);
   GrTextXY(x,y+dy*iy,buf,ColorSYel,ColorBG);
  }
  else
  {
  if (((ShipRelWaypointX(id)==0)&&(ShipRelWaypointY(id)==0))
     ||(ShipSpeed(id)==0))
  sprintf(buf,"Not moving");
  else
 /* Handle enemy ship too */
  if (LocalShips[id].infotype!=SHIPS_FULL)
  {
    if (LocalShips[id].heading!=65535)
    sprintf(buf,"Heading : %3d",LocalShips[id].heading);
    else
    sprintf(buf,"            ");
  }
  else
   sprintf(buf,"Dest : %s",WhereItIs(ShipWaypointX(id),ShipWaypointY(id)));
  GrTextXY(x,y+dy*iy,buf,ColorSI,ColorBG);
  }
  iy++;
  dis =  sqrt(ShipRelWaypointX(id)*ShipRelWaypointX(id)+
              ShipRelWaypointY(id)*ShipRelWaypointY(id));
  if (ShipIsGravitonic(id))
  ETA = ceil ( dis /(float)ShipSpeed(id)/(float)ShipSpeed(id)/2);
  else
  ETA = ceil ( dis /(float)ShipSpeed(id)/(float)ShipSpeed(id));
  if (LocalShips[id].infotype!=SHIPS_FULL) ETA=0;

  ShipWillHYP=False;
  if ((LocalShips[id].infotype==SHIPS_FULL)&&
      (ShipCanHyperwarp(id))&&
      (dis>20.0)&&
      (strcmp(ShipFC(id,NULL),"HYP")==0)&&
 //     (ShipCargo(id,NEUTRONIUM)>=50.0)&&
      (ShipSpeed(id)>0) )
                        ShipWillHYP=True;

  if (ShipWillHYP) ETA=1;

  GrTextXY(x,y+dy*iy,"Warp : ",ColorSI,ColorBG);
  sprintf(buf,"%d",ShipSpeed(id));
  GrTextXY(x+dx*8,y+dy*iy,buf,ColorSI,ColorBG);
// draw max warp when ship is damage
  if ((LocalShips[id].infotype==SHIPS_FULL)&&(ShipDamage(id)!=0))
  {
    repair=0;
    if (ShipCargo(id,SUPPLIES))
    {
      repair = floor((double)ShipCargo(id,SUPPLIES)/5.0);
      if (repair>=ShipDamage(id))
        repair =ShipDamage(id);
    }
    if (repair!=ShipDamage(id))
    {
    if (gPconfigInfo->PlayerRace[ShipOwner(id)]==Gorn)
        mwarp = 15 - floor((ShipDamage(id)-repair)/10.0);
    else
        mwarp = 10 - floor((ShipDamage(id)-repair)/10.0);

    mwarp= min(9,mwarp);
    sprintf(buf,"(%d)",mwarp);
    GrTextXY(x+dx*9,y+dy*iy,buf,ColorSYel,ColorBG);
    }
  }

  if (ETA)
  {
  GrTextXY(x+dx*13,y+dy*iy,"ETA: ",ColorSI,ColorBG);
  GrTextXY(x+dx*18,y+dy*iy,ShowInt2(ETA),ColorSI,ColorBG);
  }
  iy++;
/* No more common things */
  if (LocalShips[id].infotype!=SHIPS_FULL)
  {
  iy++;
  mass = LocalShips[id].mass;

  maxcargo = mass - HullMass(ShipHull(id));
  mincargo = mass - HullMass(ShipHull(id))
                  - HullTubeNumber(ShipHull(id))*TubeMaxMass
                  - HullBeamNumber(ShipHull(id))*BeamMaxMass;
  if (mincargo<0) mincargo=0;
  GrTextXY(x,y+dy*iy,"Ne+Crgo:",ColorSI,ColorBG);
  if (mincargo==maxcargo)
    sprintf(buf,"%d kt",mincargo);
  else
    sprintf(buf,"%d-%d kt",mincargo,maxcargo);
  GrTextXY(x+dx*8,y+dy*iy,buf,ColorSI,ColorBG);
  iy++;
  sprintf(buf,"Mass: %d kt",mass);
  GrTextXY(x,y+dy*iy,buf,ColorSI,ColorBG);
  iy++;
  iy++;

  ShowHullInfo(x,y+FontDY*iy,ddx,FontDY*10,id,ColorSI);

  GrResetClipBox();
  return False;
  }

  mass = ShipTotalMass(id);
  if (ShipIsGravitonic(id))
  fc   = (double)EngFuelConsumption(ShipEngine(id),ShipSpeed(id))
        /(double)ShipSpeed(id)/(double)ShipSpeed(id)/200000.0;
  else
  fc   = (double)EngFuelConsumption(ShipEngine(id),ShipSpeed(id))
        /(double)ShipSpeed(id)/(double)ShipSpeed(id)/100000.0;

  burn = floor(ERND(ShipTravelMass(id)/10.0)*10.0*fc*floor(dis) );

  // special case, ship can hyperjump and its jumping.
  if (ShipWillHYP) burn=50;

  sprintf(buf,"Fuel : %d/%d kt",ShipCargo(id,NEUTRONIUM),HullFuelCapacity(ShipHull(id)));
  GrTextXY(x,y+dy*iy,buf,ColorSI,ColorBG);
  iy++;
  sprintf(buf,"Burn : ");
  GrTextXY(x,y+dy*iy,buf,ColorSI,ColorBG);
  sprintf(buf,"%d kt",burn);
  if (burn>ShipCargo(id,NEUTRONIUM))
  GrTextXY(x+dx*7,y+dy*iy,buf,ColorSRed,ColorBG);
    else
  GrTextXY(x+dx*7,y+dy*iy,buf,ColorSI,ColorBG);
  iy++;

  sprintf(buf,"Mass : %d kt",mass);
  GrTextXY(x,y+dy*iy,buf,ColorSI,ColorBG);
  iy++;

  GrTextXY(x,y+dy*iy,"FCode : ",ColorSI,ColorBG);
  sprintf(buf,"%s",ShipFC(id,NULL));
  if (IsShipFCSpecial(ShipFC(id,NULL)))
    GrTextXY(x+dx*8,y+dy*iy,buf,ColorSGre,ColorBG);
  else
    GrTextXY(x+dx*8,y+dy*iy,buf,ColorSI,ColorBG);
  iy++;

  GrTextXY(x,y+dy*iy,"Crew : ",ColorSI,ColorBG);
  sprintf(buf,"%d",ShipCrew(id));
  if (HullCrewComplement(ShipHull(id))==ShipCrew(id))
  GrTextXY(x+dx*7,y+dy*iy,buf,ColorSI,ColorBG);
  else
  GrTextXY(x+dx*7,y+dy*iy,buf,ColorSYel,ColorBG);

  iy++;

  GrTextXY(x,y+dy*iy,"Damage : ",ColorSI,ColorBG);
  if (ShipDamage(id))
  {
    sprintf(buf,"%d%%",ShipDamage(id));
    GrTextXY(x+9*dx,y+dy*iy,buf,ColorSRed,ColorBG);
    if (ShipCargo(id,SUPPLIES))
    {
      repair = floor((double)ShipCargo(id,SUPPLIES)/5.0);
      if (repair>=ShipDamage(id))
        GrTextXY(x+12*dx,y+dy*iy,"(fixed)",ColorSGre,ColorBG);
      else
      {
        sprintf(buf,"(%d%%)",ShipDamage(id)-repair);
        GrTextXY(x+13*dx,y+dy*iy,buf,ColorSYel,ColorBG);
      }
    }
  }
  iy++;

  GrTextXY(x,y+dy*iy,"Misn : ",ColorSI,ColorBG);
  if (ShipMission(id)==7)
  {
    sprintf(buf,"Tow %d",ShipTowTarget(id));
    GrTextXY(x+7*dx,y+dy*iy,buf,ColorSYel,ColorBG);
  }
  else
  if (ShipMission(id)==8)
  {
    sprintf(buf,"Intercept %d",ShipInterceptTarget(id));
    GrTextXY(x+7*dx,y+dy*iy,buf,ColorSYel,ColorBG);
  }
  else


  if (ShipMission(id)==9)
  {
  if (IsShipRemoteControlled(id))
    {
      sprintf(buf,"%s",ShipSpecialMission(ShipOwner(id)));
      GrTextXY(x+7*dx,y+dy*iy,buf,ColorSGre,ColorBG);
    }
    else
    {
      sprintf(buf,"%s",ShipSpecialMission(ShipOwner(id)));
      GrTextXY(x+7*dx,y+dy*iy,buf,ColorSI,ColorBG);
    }
  }
  else
  {
  if (ShipMission(id))
      sprintf(buf,"%s",ShipMissionName(id));
  else
      strcpy(buf,"");
  GrTextXY(x+7*dx,y+dy*iy,buf,ColorSI,ColorBG);
  }
  iy++;
  /* more info on ship mission */

  if (ShipMission(id)>=gPconfigInfo->ExtMissionsStartAt)
   sprintf(buf,"%s",ShipMissionExt(id));
  else
   sprintf(buf," ");
  GrTextXY(x+7*dx,y+dy*iy,buf,ColorSI,ColorBG);
  iy++;

  if (ShipEnemy(id))
  sprintf(buf,"Enemy : %s",RaceNameAdjective(ShipEnemy(id),NULL));
  else
  sprintf(buf,"Enemy : ");
  GrTextXY(x,y+dy*iy,buf,ColorSI,ColorBG);
  iy++;
  sprintf(buf,"Equipment : ");
  GrTextXY(x,y+dy*iy,buf,ColorSI,ColorBG);
  iy++;
  if (HullEngineNumber(ShipHull(id))>1)
  sprintf(buf," %s",EngineName(ShipEngine(id),buf1));
  else
  sprintf(buf," 1 %s",EngineName(ShipEngine(id),buf1));
  GrTextXY(x,y+dy*iy,buf,ColorSI,ColorBG);
  iy++;
  if (ShipBeamNumber(id)>0)
  sprintf(buf," %d %s",ShipBeamNumber(id),BeamName(ShipBeamType(id),buf1));
  else
  sprintf(buf," No main weapons");
  GrTextXY(x,y+dy*iy,buf,ColorSI,ColorBG);
  iy++;
  if (ShipTubeNumber(id)>0)
  sprintf(buf," %d %s",ShipTubeNumber(id),TorpName(ShipTorpType(id),buf1));
  else
  if (ShipBays(id)>0)
  sprintf(buf," %d fighter bays",ShipBays(id));
  else
  sprintf(buf," No bays/lunchers");
  GrTextXY(x,y+dy*iy,buf,ColorSI,ColorBG);
  iy++;
  iy++;
  mass=ShipCargoMass(id);
  sprintf(buf,"Cargo : %d/%d",mass,HullCargoCapacity(ShipHull(id)));
  GrTextXY(x,y+dy*iy,buf,ColorSI,ColorBG);
  iy++;

  GrTextXY(x,y+dy*iy,"Ne: ",ColorSI,ColorBG);
  GrTextXY(x+dx*4,y+dy*iy,ShowInt4(ShipCargo(id,NEUTRONIUM)),ColorSI,ColorBG);
  GrTextXY(x+dx*10,y+dy*iy,ShowInt4(ShipDump(id,NEUTRONIUM)),ColorSP,ColorBG);
  GrTextXY(x+dx*15,y+dy*iy,ShowInt4(ShipTransfer(id,NEUTRONIUM)),ColorST,ColorBG);
  iy++;
  GrTextXY(x,y+dy*iy,"Tr: ",ColorSI,ColorBG);
  GrTextXY(x+dx*4,y+dy*iy,ShowInt4(ShipCargo(id,TRITANIUM)),ColorSI,ColorBG);
  GrTextXY(x+dx*10,y+dy*iy,ShowInt4(ShipDump(id,TRITANIUM)),ColorSP,ColorBG);
  GrTextXY(x+dx*15,y+dy*iy,ShowInt4(ShipTransfer(id,TRITANIUM)),ColorST,ColorBG);
  iy++;
  GrTextXY(x,y+dy*iy,"Du: ",ColorSI,ColorBG);
  GrTextXY(x+dx*4,y+dy*iy,ShowInt4(ShipCargo(id,DURANIUM)),ColorSI,ColorBG);
  GrTextXY(x+dx*10,y+dy*iy,ShowInt4(ShipDump(id,DURANIUM)),ColorSP,ColorBG);
  GrTextXY(x+dx*15,y+dy*iy,ShowInt4(ShipTransfer(id,DURANIUM)),ColorST,ColorBG);
  iy++;
  GrTextXY(x,y+dy*iy,"Mo: ",ColorSI,ColorBG);
  GrTextXY(x+dx*4,y+dy*iy,ShowInt4(ShipCargo(id,MOLYBDENUM)),ColorSI,ColorBG);
  GrTextXY(x+dx*10,y+dy*iy,ShowInt4(ShipDump(id,MOLYBDENUM)),ColorSP,ColorBG);
  GrTextXY(x+dx*15,y+dy*iy,ShowInt4(ShipTransfer(id,MOLYBDENUM)),ColorST,ColorBG);
  iy++;
  GrTextXY(x,y+dy*iy,"Sp: ",ColorSI,ColorBG);
  GrTextXY(x+dx*4,y+dy*iy,ShowInt4(ShipCargo(id,SUPPLIES)),ColorSI,ColorBG);
  GrTextXY(x+dx*10,y+dy*iy,ShowInt4(ShipDump(id,SUPPLIES)),ColorSP,ColorBG);
  GrTextXY(x+dx*15,y+dy*iy,ShowInt4(ShipTransfer(id,SUPPLIES)),ColorST,ColorBG);
  iy++;
  GrTextXY(x,y+dy*iy,"Cl: ",ColorSI,ColorBG);
  GrTextXY(x+dx*4,y+dy*iy,ShowInt4(ShipCargo(id,COLONISTS)),ColorSI,ColorBG);
  GrTextXY(x+dx*10,y+dy*iy,ShowInt4(ShipDump(id,COLONISTS)),ColorSP,ColorBG);
  GrTextXY(x+dx*15,y+dy*iy,ShowInt4(ShipTransfer(id,COLONISTS)),ColorST,ColorBG);
  iy++;
  GrTextXY(x,y+dy*iy,"$$: ",ColorSI,ColorBG);
  GrTextXY(x+dx*4,y+dy*iy,ShowInt4(ShipCargo(id,CREDITS)),ColorSI,ColorBG);
  iy++;
  if (ShipBays(id)>0)
  {
  GrTextXY(x,y+dy*iy,"Fg: ",ColorSI,ColorBG);
  GrTextXY(x+dx*4,y+dy*iy,ShowInt4(ShipAmmunition(id)),ColorSI,ColorBG);
  }
  if (ShipTubeNumber(id)>0)
  {
  GrTextXY(x,y+dy*iy,"Tr: ",ColorSI,ColorBG);
  GrTextXY(x+dx*4,y+dy*iy,ShowInt4(ShipAmmunition(id)),ColorSI,ColorBG);
  }
  iy++;
  iy++;
  if (ShipTransferShip(id))
  {
  sprintf(buf,"Transfer to %d",ShipTransferShip(id));
  GrTextXY(x,y+dy*iy,buf,ColorST,ColorBG);
  }

  iy++;
  if (ShipDumpPlanet(id))
  {
  sprintf(buf,"Transfer to %d",ShipDumpPlanet(id));
  GrTextXY(x,y+dy*iy,buf,ColorSP,ColorBG);
  }

  GrResetClipBox();
  return True;
}

static char *ShowInt4(int value)
{
    static char show[8];

    if (value==0)   strcpy(show,"    ");
    else
    if (value>9999) strcpy(show,"++++");
    else
    sprintf(show,"%-4d",value);

    return show;
}

static char *ShowInt2(int value)
{
    static char show[8];

    if (value==0)   strcpy(show,"    ");
    else
    if (value>99) strcpy(show,"++");
    else
    sprintf(show,"%-2d",value);

    return show;
}


int ERND(double value)
{
   double base;
   double fraction;

    fraction=modf(value,&base);

    if ( (int)(base)/2*2==(int)(base))
    { 
    if (fraction<=0.5) return base;
    else               return base+1;
    }
    else
    {
    if (fraction<0.5) return base;
    else              return base+1;
    }
}

int RND(double value)
{
   double base;
   double fraction;

    fraction=modf(value,&base);

    if (fraction<0.5) return base;
    else return base+1;
}

static char *ShipMissionName(Uns16 sID)
{

  static char miss[40];
  static char spm[40];
  int mission,i;
  Uns16 tmprace;

  mission=ShipMission(sID);
  tmprace   =gPconfigInfo->PlayerRace[ShipOwner(sID)];

  switch (tmprace)
  {
   case 1: strcpy(spm,"Super Refit"); break; 	
   case 2: strcpy(spm,"Hissssss!"); break;
   case 3: strcpy(spm,"Super Spy"); break;
   case 4: strcpy(spm,"Pillage"); break;
   case 5: strcpy(spm,"Rob Ship"); break;
   case 6: strcpy(spm,"Repair Self"); break;
   case 7: strcpy(spm,"Web Mines"); break;
   case 8: strcpy(spm,"Dark Sense"); break;
   case 9: strcpy(spm,"Bld Fighters"); break;
   case 10: strcpy(spm,"Grnd Attack"); break;
   case 11: strcpy(spm,"Bld Fighters"); break;
  }

  switch (mission)
  {
   case 0: strcpy(miss,"NoMission"); break;
   case 1: strcpy(miss,"Exploration"); break;
   case 2: strcpy(miss,"Mine Sweep"); break;
   case 3: strcpy(miss,"Lay Mines"); break;
   case 4: strcpy(miss,"KILL!!!"); break;
   case 5: strcpy(miss,"Sensor Sweep"); break;
   case 6: strcpy(miss,"Colonize"); break;
   case 7: strcpy(miss,"Tow"); break;
   case 8: strcpy(miss,"Intercept"); break;
   case 9: strcpy(miss,spm); break;
   case 10: strcpy(miss,"Cloak"); break;
   case 11: strcpy(miss,"Beam up Fuel"); break;
   case 12: strcpy(miss,"Beam up Du"); break;
   case 13: strcpy(miss,"Beam up Tr"); break;
   case 14: strcpy(miss,"Beam up Mo"); break;
   case 15: strcpy(miss,"Beam up Supp"); break;
  }

  for (i=0;i<ExtMission_no;i++)
   if (mission==i+gPconfigInfo->ExtMissionsStartAt)
     strcpy(miss,ExtMission[i].name);

  return miss;
}

static char *ShipSpecialMission(int tmprace)
{

  static char spm[40];

  switch (gPconfigInfo->PlayerRace[tmprace])
  {
   case 1: strcpy(spm,"Super Refit"); break; 	
   case 2: strcpy(spm,"Hissssss!"); break;
   case 3: strcpy(spm,"Super Spy"); break;
   case 4: strcpy(spm,"Pillage"); break;
   case 5: strcpy(spm,"Rob Ship"); break;
   case 6: strcpy(spm,"Repair Self"); break;
   case 7: strcpy(spm,"Web Mines"); break;
   case 8: strcpy(spm,"Dark Sense"); break;
   case 9: strcpy(spm,"Bld Fighters"); break;
   case 10: strcpy(spm,"Grnd Attack"); break;
   case 11: strcpy(spm,"Bld Fighters"); break;
  }
  return spm;
}

static char *ShipMissionExt(int sID)
{
  static char spm[40];
  int mission;
  char s[40];

  strcpy(spm,"");
  mission=ShipMission(sID)-gPconfigInfo->ExtMissionsStartAt;
   if (mission>=0)
   {
     if (ExtMission[mission].yesint)
     {
     sprintf(s,"I:%d",ShipInterceptTarget(sID));
     strcat(spm,s);
     }
     if (ExtMission[mission].yestow)
     {
     sprintf(s," T:%d",ShipTowTarget(sID));
     strcat(spm,s);
     }
   }

   return spm;
}


Uns16 ShipTravelMass(Uns16 id)
{
  Uns16 mass=0;

  mass = ShipTotalMass(id);

  if (ShipMission(id)==7)
       if ((IsShipExist(ShipTowTarget(id)))&&(LocalShips[ShipTowTarget(id)].infotype>SHIPS_NOINFO))
       mass+=ShipTotalMass(ShipTowTarget(id));

  return mass;
}

Uns16 ShipTotalMass(Uns16 id)
{
  Uns16 mass=0;

  if (!IsShipExist(id)) return 0;
  if (LocalShips[id].infotype==SHIPS_PARTIAL) return LocalShips[id].mass;

  mass = HullMass(ShipHull(id));

  if (ShipBeamNumber(id)>0) mass += ShipBeamNumber(id)*BeamMass(ShipBeamType(id));
  if (ShipTubeNumber(id)>0) mass += ShipTubeNumber(id)*TorpTubeMass(ShipTorpType(id));

  mass += ShipCargo(id,NEUTRONIUM);
  mass += ShipCargo(id,TRITANIUM);
  mass += ShipCargo(id,DURANIUM);
  mass += ShipCargo(id,MOLYBDENUM);
  mass += ShipCargo(id,COLONISTS);
  mass += ShipCargo(id,SUPPLIES);
  mass += ShipAmmunition(id);

  return mass;
}

Uns16 ShipCargoMass(Uns16 id)
{
  Uns16 mass=0;

  mass  = ShipCargo(id,TRITANIUM);
  mass += ShipCargo(id,DURANIUM);
  mass += ShipCargo(id,MOLYBDENUM);
  mass += ShipCargo(id,COLONISTS);
  mass += ShipCargo(id,SUPPLIES);
  mass += ShipAmmunition(id);

  return mass;
}

static char *WhereItIs(int x,int y)
{
 static char Name[40]="No loaded\0";
 int i,lx,ly;
 int pid=0;
 double dist;
 Boolean Near,Found;

 lx=x;
 if (lx<gPconfigInfo->WraparoundRectangle[0]) lx +=2000;
 if (lx>gPconfigInfo->WraparoundRectangle[2]) lx -=2000;

 ly=y;
 if (ly<gPconfigInfo->WraparoundRectangle[1]) ly +=2000;
 if (ly>gPconfigInfo->WraparoundRectangle[3]) ly -=2000;

 Found=False;
 for(i=1;i<=500;i++)
   if (IsPlanetExist(i))
   {
    dist=Distance(lx,ly,PlanetLocationX(i),PlanetLocationY(i));
    if (dist==0.0)
    { pid = i; Near=False; Found=True; break; }
    if (dist<=3.0)
    { pid = i; Near=True;  Found=True; break; }
   }

 if (Found)
 {
   if (Near) sprintf(Name,"~%s",PlanetName(pid,NULL));
   else sprintf(Name,"%s",PlanetName(pid,NULL));
 }
 else
   sprintf(Name,"(%4d,%4d)",lx,ly);

 return Name;
}


int FindRemoteFromShipName(char *name)
{
  int i,tmprace;

  for (i=19;i>0;i--)
   if (((name[i]=='A')||(name[i]=='B')||(name[i]>='1')||(name[i]<='9'))
      &&(name[i-1]=='*'))
   {
     tmprace=0;
     if (name[i]>='1') tmprace=name[i]-'1'+1;
     if (name[i]=='A') tmprace=10;
     if (name[i]=='B') tmprace=11;

     return tmprace;
   }
 return 0;
}

int FindShipTowee(Uns16 sID)
{

    Uns16 i;

    for (i=1;i<=500;i++)
    if ((IsShipExist(i))&&(LocalShips[i].infotype==SHIPS_FULL))
     if (ShipMission(i)==7)
      if (ShipTowTarget(i)==sID)
       return i;

    return 0;
}

Boolean IsShipFCSpecial(const char *fc)
{
   if (strcmp(fc,"mkt")==0) return True;
   if (strcmp(fc,"mdh")==0) return True;
   if (strcmp(fc,"mdq")==0) return True;
   if (strcmp(fc,"msc")==0) return True;
   if (strcmp(fc,"alt")==0) return True;
   if (strcmp(fc,"ald")==0) return True;
   if (strcmp(fc,"alm")==0) return True;
   if (strcmp(fc,"NAL")==0) return True;
   if (strcmp(fc,"HYP")==0) return True;
   if (strcmp(fc,"NTP")==0) return True;
   if (strcmp(fc,"WRS")==0) return True;
   if (strcmp(fc,"WRT")==0) return True;
   if (strcmp(fc,"lfm")==0) return True;
   if (strcmp(fc,"bdm")==0) return True;
   if (strcmp(fc,"nat")==0) return True;
   if (strcmp(fc,"nad")==0) return True;
   if (strcmp(fc,"nam")==0) return True;
   if (strcmp(fc,"cln")==0) return True;
   if (strcmp(fc,"btt")==0) return True;
   if (strcmp(fc,"btf")==0) return True;
   if (strcmp(fc,"btm")==0) return True;
   if (strcmp(fc,"pop")==0) return True;
   if (strcmp(fc,"trg")==0) return True;
   if (strcmp(fc,"nbr")==0) return True;
   if (strcmp(fc,"gs1")==0) return True;
   if (strcmp(fc,"gs2")==0) return True;
   if (strcmp(fc,"gs3")==0) return True;
   if (strcmp(fc,"gs4")==0) return True;
   if (strcmp(fc,"gs5")==0) return True;
   if (strcmp(fc,"gs6")==0) return True;
   if (strcmp(fc,"gs7")==0) return True;
   if (strcmp(fc,"gs8")==0) return True;
   if (strcmp(fc,"gs9")==0) return True;
   if (strcmp(fc,"gsa")==0) return True;
   if (strcmp(fc,"gsb")==0) return True;
   return False;
}

Boolean IsPlanetFCSpecial(const char *fc)
{
   if (strcmp(fc,"NUK")==0) return True;
   if (strcmp(fc,"ATT")==0) return True;
   if (strcmp(fc,"dmp")==0) return True;
   if (strcmp(fc,"bum")==0) return True;
   if (strcmp(fc,"con")==0) return True;
   if (strcmp(fc,"PB1")==0) return True;
   if (strcmp(fc,"PB2")==0) return True;
   if (strcmp(fc,"PB3")==0) return True;
   if (strcmp(fc,"PB4")==0) return True;
   if (strcmp(fc,"PB5")==0) return True;
   if (strcmp(fc,"PB6")==0) return True;
   if (strcmp(fc,"PB7")==0) return True;
   if (strcmp(fc,"PB8")==0) return True;
   if (strcmp(fc,"PB9")==0) return True;
   return False;
}

int WrapXDist(int x1,int x2)
{
     int t,x;

     x = -(x1 - x2);

     if (gPconfigInfo->AllowWraparoundMap)
     {
     if (x>=0)
     t= -(gPconfigInfo->WraparoundRectangle[2]-gPconfigInfo->WraparoundRectangle[0])-(x1-x2);
     else
     t= (gPconfigInfo->WraparoundRectangle[2]-gPconfigInfo->WraparoundRectangle[0])-(x1-x2);
     x=amin(x,t);
     }
     return x;
}

int WrapYDist(int x1,int x2)
{
     int t,x;

     x = -(x1 - x2);

     if (gPconfigInfo->AllowWraparoundMap)
     {
     if (x>=0)
     t= -(gPconfigInfo->WraparoundRectangle[3]-gPconfigInfo->WraparoundRectangle[1])-(x1-x2);
     else
     t= (gPconfigInfo->WraparoundRectangle[3]-gPconfigInfo->WraparoundRectangle[1])-(x1-x2);
     x=amin(x,t);
     }
     return -x;
}

int amin(int v1, int v2)
{
   return ( (abs(v1) < abs(v2)) ? v1 : v2);
}

int MapX(int x)
{
   int dx=(gPconfigInfo->WraparoundRectangle[2]-gPconfigInfo->WraparoundRectangle[0]);
   if (x>gPconfigInfo->WraparoundRectangle[2]) return x-dx;
   if (x<gPconfigInfo->WraparoundRectangle[0]) return x+dx;
   return x;
}

int MapY(int x)
{
   int dx=(gPconfigInfo->WraparoundRectangle[3]-gPconfigInfo->WraparoundRectangle[1]);
   if (x>gPconfigInfo->WraparoundRectangle[3]) return x-dx;
   if (x<gPconfigInfo->WraparoundRectangle[1]) return x+dx;
   return x;
}

double          DistWrap(double x1,double y1, double x2, double y2)
{
     double          x,
		     y,
		     t,
		     out,
                     dx,dy ;

     dx = gPconfigInfo->WraparoundRectangle[2]-gPconfigInfo->WraparoundRectangle[0];
     dy = gPconfigInfo->WraparoundRectangle[3]-gPconfigInfo->WraparoundRectangle[1];

     x = fabs(x1 - x2);
     y = fabs(y1 - y2);

     t= dx-fabs(x1-x2);
     x=min(x,t);

     t= dy-fabs(y1-y2);
     y=min(y,t);

     out = x*x;
     out += y*y;
     out = sqrt(out);

     return (out);
}
double min(double v1, double v2)
{
   return ( (v1 < v2) ? v1 : v2);
}
double max(double v1, double v2)
{
   return ( (v1 > v2) ? v1 : v2);
}


void LoadAllyFromUtil(int race)
{
 FILE *f;
 char s[80],buf[2000];
 int j,i;

 struct record_head head;
 struct util_ally_rec ally;

 sprintf(s,"util%d.dat",race);
 f=OpenInputFile(s,GAME_DIR_ONLY|NO_MISSING_ERROR);
 if (f==NULL) return;

 while (!feof(f))
 {
   fread(&head,sizeof(head),1,f);
   if (!feof(f))
   {
// ally record
   if (head.recordtype==22)
   {
     fread(&ally,sizeof(ally),1,f);
     Info("Reading ally from util%d.dat",race);
     head.recordlen=0;

   /* unconditional aliance offered */
     for (j=1;j<12;j++)
      if (j!=race)
       if ((ally.OfferTo[j-1]&0x20)!=0)
      {
       AllyAddRequest(race,j);
       if ((ally.OfferTo[j-1]&0x01)!=0) PutPlayerAllowsAlly(race,j,ALLY_SHIPS,ALLY_STATE_ON);
       if ((ally.OfferTo[j-1]&0x02)!=0) PutPlayerAllowsAlly(race,j,ALLY_PLANETS,ALLY_STATE_ON);
       if ((ally.OfferTo[j-1]&0x04)!=0) PutPlayerAllowsAlly(race,j,ALLY_MINES,ALLY_STATE_ON);
       if ((ally.OfferTo[j-1]&0x08)!=0) PutPlayerAllowsAlly(race,j,ALLY_COMBAT,ALLY_STATE_ON);
       if ((ally.OfferTo[j-1]&0x10)!=0) PutPlayerAllowsAlly(race,j,ALLY_VISION,ALLY_STATE_ON);
      }

  /* unconditional aliance recived */
     for (j=1;j<12;j++)
      if (j!=race)
       if ((ally.OfferFrom[j-1]&0x20)!=0)
      {
       AllyAddRequest(j,race);
       if ((ally.OfferFrom[j-1]&0x01)!=0) PutPlayerAllowsAlly(j,race,ALLY_SHIPS,ALLY_STATE_ON);
       if ((ally.OfferFrom[j-1]&0x02)!=0) PutPlayerAllowsAlly(j,race,ALLY_PLANETS,ALLY_STATE_ON);
       if ((ally.OfferFrom[j-1]&0x04)!=0) PutPlayerAllowsAlly(j,race,ALLY_MINES,ALLY_STATE_ON);
       if ((ally.OfferFrom[j-1]&0x08)!=0) PutPlayerAllowsAlly(j,race,ALLY_COMBAT,ALLY_STATE_ON);
       if ((ally.OfferFrom[j-1]&0x10)!=0) PutPlayerAllowsAlly(j,race,ALLY_VISION,ALLY_STATE_ON);
      }

   /* conditional aliance offered */
     for (j=1;j<12;j++)
      if (j!=race)
       if ((ally.ConditionalTo[j-1]&0x20)!=0)
      {
       AllyAddRequest(race,j);
       if ((ally.ConditionalTo[j-1]&0x01)!=0) PutPlayerAllowsAlly(race,j,ALLY_SHIPS,ALLY_STATE_CONDITIONAL);
       if ((ally.ConditionalTo[j-1]&0x02)!=0) PutPlayerAllowsAlly(race,j,ALLY_PLANETS,ALLY_STATE_CONDITIONAL);
       if ((ally.ConditionalTo[j-1]&0x04)!=0) PutPlayerAllowsAlly(race,j,ALLY_MINES,ALLY_STATE_CONDITIONAL);
       if ((ally.ConditionalTo[j-1]&0x08)!=0) PutPlayerAllowsAlly(race,j,ALLY_COMBAT,ALLY_STATE_CONDITIONAL);
       if ((ally.ConditionalTo[j-1]&0x10)!=0) PutPlayerAllowsAlly(race,j,ALLY_VISION,ALLY_STATE_CONDITIONAL);
      }

  /* conditional aliance recived */
     for (j=1;j<12;j++)
      if (j!=race)
       if ((ally.ConditionalFrom[j-1]&0x20)!=0)
      {
       AllyAddRequest(j,race);
       if ((ally.ConditionalFrom[j-1]&0x01)!=0) PutPlayerAllowsAlly(j,race,ALLY_SHIPS,ALLY_STATE_CONDITIONAL);
       if ((ally.ConditionalFrom[j-1]&0x02)!=0) PutPlayerAllowsAlly(j,race,ALLY_PLANETS,ALLY_STATE_CONDITIONAL);
       if ((ally.ConditionalFrom[j-1]&0x04)!=0) PutPlayerAllowsAlly(j,race,ALLY_MINES,ALLY_STATE_CONDITIONAL);
       if ((ally.ConditionalFrom[j-1]&0x08)!=0) PutPlayerAllowsAlly(j,race,ALLY_COMBAT,ALLY_STATE_CONDITIONAL);
       if ((ally.ConditionalFrom[j-1]&0x10)!=0) PutPlayerAllowsAlly(j,race,ALLY_VISION,ALLY_STATE_CONDITIONAL);
      }

  }

  i=head.recordlen;
  while (i>0)
  {
   if (i>2000)   fread(buf,2000,1,f);
   else          fread(buf,i,1,f);
   i-=2000;
  }
  } // if feof
 } // while feof

}


void  ShowObjectsAtPos(int x,int y, int ddx, int ddy, int race)
{
   Uns16 j;
   GrColor C= GrAllocColor(0x00,0x00,0x00);
   char s[40];
   char s1[40], *s2;
   Uns16 DrawSX,DrawSY;

// ddx, ddy now not used

   DrawSX=x;
   DrawSY=y;

   for (j=0;j<objects_in_pos_no;j++)
   {
     if (objects_in_pos[j].type==OBJECT_SHIP)
     {
     if (objects_in_pos[j].reowner!=0)
     {
     if (objects_in_pos[j].reowner==race)
        C=ColorSMRace;
     else
        if (PlayerAllowsAlly(race,objects_in_pos[j].reowner,ALLY_SHIPS))
           C=ColorSMAlly;
        else
           C=ColorSMEnemy;
     }
     else
     {
       if (objects_in_pos[j].owner==race)
             C=ColorSMRace;
       else
            if (PlayerAllowsAlly(race,objects_in_pos[j].owner,ALLY_SHIPS))
                 C=ColorSMAlly;
            else
                 C=ColorSMEnemy;
      }
      /* Display name on list */
      if (C==ColorSMEnemy)
      {
        HullName( ShipHull(objects_in_pos[j].id),s1);
        strncpy(s,s1,20);
        s[20]=0;
        }
      else
        s2=ShipName(objects_in_pos[j].id,s);
      GrTextXY(DrawSX,DrawSY+j*FontDY,s,C,ColorBG);
    } // object ship

    if (objects_in_pos[j].type==OBJECT_MINEFIELD)
    {
      sprintf(s1,"Minefield ID:%d",objects_in_pos[j].id);
      sprintf(s,"%-20s",s1);
      s[20]=0;
      GrTextXY(DrawSX,DrawSY+j*FontDY,s,MinefieldColor[objects_in_pos[j].id],ColorBG);
    }

   } // all objects
}

void LoadMissionIniFile(void)
{
  FILE *f;
  char buf[200];
  int m;
  char tow[80],inter[80];
  Boolean yesint,yestow;

  f=OpenInputFile("mission.ini",GAME_DIR_ONLY|NO_MISSING_ERROR|TEXT_MODE);
  if (f==NULL) return;
  Info("Loading mission.ini file.");
  fgets(buf,sizeof(buf),f);
  while (!feof(f))
  {
    sscanf(buf,"%d",&m);
    inter[0]=0;
    tow[0]=0;
    buf[strlen(buf)-1]=0;
    if ((buf[strlen(buf)-1]=='#')||(buf[strlen(buf)-1]=='*'))
    {
      yestow=False;
      if (buf[strlen(buf)-1]=='#')
      {
        buf[strlen(buf)-1]=0;
        if (strstr(buf,"T:")!=NULL) sscanf(strstr(buf,"T:"),"%s",tow);
        yestow=True;
      }
      yesint=False;
      if (buf[strlen(buf)-1]=='*')
      {
        buf[strlen(buf)-1]=0;
        if (strstr(buf,"I:")!=NULL) sscanf(strstr(buf,"I:"),"%s",inter);
        yesint=True;

      }
     if (strstr(inter,")")!=NULL) strstr(inter,")")[0]=0;
     if (strstr(tow,")")  !=NULL) strstr(tow,")")[0]=0;
     if (strstr(buf,"(")  !=NULL) strstr(buf,"(")[0]=0;

     if (yestow)
        strcpy(ExtMission[m-gPconfigInfo->ExtMissionsStartAt].tow,&tow[2]);
     ExtMission[m-gPconfigInfo->ExtMissionsStartAt].yestow=yestow;

     if (yesint)
        strcpy(ExtMission[m-gPconfigInfo->ExtMissionsStartAt].inter,&inter[2]);
     ExtMission[m-gPconfigInfo->ExtMissionsStartAt].yesint=yesint;

     }
    else
    {
     ExtMission[m-gPconfigInfo->ExtMissionsStartAt].yestow=False;
     ExtMission[m-gPconfigInfo->ExtMissionsStartAt].yesint=False;
    }
   strcpy(ExtMission[m-gPconfigInfo->ExtMissionsStartAt].name,&buf[4]);
   ExtMission_no++;

   if (!feof(f)) fgets(buf,sizeof(buf),f);
  }
  return;
}

/* ***************************************************************** */
/*              load planet                                          */
/* ***************************************************************** */

void LoadPlanets(int tmprace)
{

   char name[20],buf_s[50];
   FILE *f;
   Uns16 howmany,i;
   Uns16 id;
   Uns16 owner;
   Uns16 buf;
   Uns32 buf32;

   sprintf(name,"pdata%d.dat",tmprace);
   f=OpenInputFile(name,GAME_DIR_ONLY);

   fread(&howmany,sizeof(howmany),1,f);
   for(i=0;i<howmany;i++)
   {
     fread(&owner,sizeof(owner),1,f);
     fread(&id,sizeof(id),1,f);
     PutPlanetOwner(id,owner);
     if (owner!=0)
       LocalPlanets[id]=PLANETS_INFO_FULL;
     else
       LocalPlanets[id]=PLANETS_INFO_UNOWNED;

     fread(buf_s,3,1,f);
     buf_s[3]=0;
     PutPlanetFC(id,buf_s);

     fread(&buf,sizeof(buf),1,f);
     PutPlanetMines(id,buf);
     fread(&buf,sizeof(buf),1,f);
     PutPlanetFactories(id,buf);
     fread(&buf,sizeof(buf),1,f);
     PutPlanetDefense(id,buf);

     fread(&buf32,sizeof(buf32),1,f);
     PutPlanetCargo(id,NEUTRONIUM,buf32);
     fread(&buf32,sizeof(buf32),1,f);
     PutPlanetCargo(id,TRITANIUM,buf32);
     fread(&buf32,sizeof(buf32),1,f);
     PutPlanetCargo(id,DURANIUM,buf32);
     fread(&buf32,sizeof(buf32),1,f);
     PutPlanetCargo(id,MOLYBDENUM,buf32);
     fread(&buf32,sizeof(buf32),1,f);
     PutPlanetCargo(id,COLONISTS,buf32*100);
     fread(&buf32,sizeof(buf32),1,f);
     PutPlanetCargo(id,SUPPLIES,buf32);
     fread(&buf32,sizeof(buf32),1,f);
     PutPlanetCargo(id,CREDITS,buf32);

     fread(&buf32,sizeof(buf32),1,f);
     PutPlanetCore(id,NEUTRONIUM,buf32);
     fread(&buf32,sizeof(buf32),1,f);
     PutPlanetCore(id,TRITANIUM,buf32);
     fread(&buf32,sizeof(buf32),1,f);
     PutPlanetCore(id,DURANIUM,buf32);
     fread(&buf32,sizeof(buf32),1,f);
     PutPlanetCore(id,MOLYBDENUM,buf32);

     fread(&buf,sizeof(buf),1,f);
     PutPlanetDensity(id,NEUTRONIUM,buf);
     fread(&buf,sizeof(buf),1,f);
     PutPlanetDensity(id,TRITANIUM,buf);
     fread(&buf,sizeof(buf),1,f);
     PutPlanetDensity(id,DURANIUM,buf);
     fread(&buf,sizeof(buf),1,f);
     PutPlanetDensity(id,MOLYBDENUM,buf);

     fread(&buf,sizeof(buf),1,f);
     PutPlanetColTax(id,buf);
     fread(&buf,sizeof(buf),1,f);
     PutPlanetNatTax(id,buf);
     fread(&buf,sizeof(buf),1,f);
     PutPlanetColHappy(id,buf);
     fread(&buf,sizeof(buf),1,f);
     PutPlanetNatHappy(id,buf);

     fread(&buf,sizeof(buf),1,f);
     PutPlanetNatGovm(id,buf);
     fread(&buf32,sizeof(buf32),1,f);
     PutPlanetNativePopulation(id,buf32*100);
     fread(&buf,sizeof(buf),1,f);
     PutPlanetNatType(id,buf);

     fread(&buf,sizeof(buf),1,f);
     PutPlanetTemp(id,buf);
     fread(&buf,sizeof(buf),1,f);
     PutPlanetBuildBase(id,buf);
   }

   if (f!=NULL) fclose(f);
}


Boolean ShowPlanet(int x,int y,int ddx, int ddy, int id)
{

  char buf[100];
  char buf1[100];
  int dx=FontDX,dy=FontDY,iy=0,nathc;
  Uns16 t1,t2,miner;
  GrColor C;


  if (!IsPlanetExist(id)) return False;

  GrSetClipBox(x, y, x+ddx, y+ddy);
  GrFilledBox (x, y, x+ddx, y+ddy,ColorBG);

  sprintf(buf,"Planet %d",id);
  GrTextXY(x,y+dy*iy,buf,ColorPt,ColorBG);
  iy++;

  sprintf(buf,"%s",PlanetName(id,NULL));
  GrTextXY(x,y+dy*iy,buf,ColorPt,ColorBG);
  iy++;

  if ( LocalPlanets[id]==PLANETS_INFO_NO_INFO)
  {
   iy++;
   GrTextXY(x,y+dy*iy,"(No data)",ColorPN,ColorBG);
   GrResetClipBox();
   return False;
  }


  sprintf(buf,"(now)");
  GrTextXY(x,y+dy*iy,buf,ColorPN,ColorBG);
  iy++;
  iy++;

  if (LocalPlanets[id]!=PLANETS_INFO_BIOSCAN)
  {
  if (PlanetOwner(id)!=0)
  RaceNameAdjective(PlanetOwner(id),buf1);
  else
  strcpy(buf1,"");
  sprintf(buf,"Race: %s",buf1);
  GrTextXY(x,y+dy*iy,buf,ColorPI,ColorBG);
  }
  iy++;

// if only sensor sweep

  if (LocalPlanets[id]==PLANETS_INFO_SENSOR)
  {
    iy=17;
    GrTextXY(x,y+dy*iy,SensorSweepString(SensorSweepActivity[id]),ColorPI,ColorBG);
    iy++;
    GrTextXY(x,y+dy*iy,"   industrial",ColorPI,ColorBG);
    iy++;
    GrTextXY(x,y+dy*iy,"        activity",ColorPI,ColorBG);
   GrResetClipBox();
    return True;
  }

 // FC - full,super spy
  if (
      ((LocalPlanets[id]&PLANETS_INFO_FULL)==PLANETS_INFO_FULL)||
      ((LocalPlanets[id]&PLANETS_INFO_SPY) ==PLANETS_INFO_SPY )||
      ((LocalPlanets[id]&PLANETS_INFO_UNOWNED)   ==PLANETS_INFO_UNOWNED)
     )
  {
  GrTextXY(x,y+dy*iy,"FCode: ",ColorPI,ColorBG);
  if (IsPlanetFCSpecial(PlanetFCode(id,NULL)))
  C=ColorPWarning;
  else
  C=ColorPI;
  sprintf(buf,"%s",PlanetFCode(id,NULL));
  GrTextXY(x+dx*7,y+dy*iy,buf,C      ,ColorBG);
  } // fc

  iy++;
  iy++;
  if (
      ((LocalPlanets[id]&PLANETS_INFO_FULL)    ==PLANETS_INFO_FULL    )||
      ((LocalPlanets[id]&PLANETS_INFO_EXPLORE) ==PLANETS_INFO_EXPLORE )||
      ((LocalPlanets[id]&PLANETS_INFO_TARGET)  ==PLANETS_INFO_TARGET  )||
      ((LocalPlanets[id]&PLANETS_INFO_BIOSCAN) ==PLANETS_INFO_BIOSCAN  )||
      ((LocalPlanets[id]&PLANETS_INFO_UNOWNED) ==PLANETS_INFO_UNOWNED)

      )
  {
  sprintf(buf,"Climate: %s %d",PlanetClimate(id),100-PlanetTemp(id));
  GrTextXY(x,y+dy*iy,buf,ColorPI,ColorBG);
  } // climate

  iy++;
  if (((LocalPlanets[id]&PLANETS_INFO_FULL)    ==PLANETS_INFO_FULL    )||
      ((LocalPlanets[id]&PLANETS_INFO_TARGET)  ==PLANETS_INFO_TARGET  )||
      ((LocalPlanets[id]&PLANETS_INFO_BIOSCAN) ==PLANETS_INFO_BIOSCAN  )||
      ((LocalPlanets[id]&PLANETS_INFO_UNOWNED) ==PLANETS_INFO_UNOWNED  )
      )
  {
  if (PlanetNatType(id)!=NoNatives)
  {
    GrTextXY(x,y+dy*iy,"Natives:",ColorPI,ColorBG);
    iy++;
    sprintf(buf,"%s",PlanetNatString(id));
    GrTextXY(x+dx,y+dy*iy,buf,ColorPI,ColorBG);
    iy++;
    if  (LocalPlanets[id]!=PLANETS_INFO_BIOSCAN )
    {
    sprintf(buf,"%s",PlanetNatGovmString(id));
    GrTextXY(x+dx,y+dy*iy,buf,ColorPI,ColorBG);
    sprintf(buf,"%3d%%",PlanetGovTaxRate(id));
    GrTextXY(x+dx*16,y+dy*iy,buf,ColorPI,ColorBG);
    }
    iy++;
    sprintf(buf,"Pop. : %ld",PlanetNativePopulation(id));
    GrTextXY(x+dx,y+dy*iy,buf,ColorPI,ColorBG);
    iy++;
    if ((LocalPlanets[id]&PLANETS_INFO_FULL)    ==PLANETS_INFO_FULL    )
    {
       if (PlanetNatHappyChange(id)<0) C=ColorPRed;
       else                            C=ColorPI;
       if (PlanetNatTax(id)>0)
       sprintf(buf,"Taxes: %d%%",PlanetNatTax(id));
       else
       sprintf(buf,"Taxes: No");
       GrTextXY(x+dx,y+dy*iy,buf,C,ColorBG);
       iy++;
       nathc=PlanetNatHappy(id);
       sprintf(buf,"Happy: %d%%",nathc);
       if (nathc<40) C=ColorPRed;
       if (nathc>=40) C=ColorPWarning;
       if (nathc>=70) C=ColorPI;
       GrTextXY(x+dx,y+dy*iy,buf,C    ,ColorBG);
       nathc=PlanetNatHappyChange(id);
       if (nathc!=0)
       {
       if (nathc>0)
       {
           sprintf(buf1,"[%+d]",nathc);
           sprintf(buf,"%-5s",buf1);
           GrTextXY(x+dx*14,y+dy*iy,buf,ColorPI,ColorBG);
       }
       else
       {
           sprintf(buf1,"[%+d]",nathc);
           sprintf(buf,"%-5s",buf1);
           GrTextXY(x+dx*14,y+dy*iy,buf,ColorPRed,ColorBG);
       }
       } // nathc !=0
    } // only full has tax rate and happy
    else
    {
      iy++;
    }
  } // natives exists
  else
  {
    iy++;
    GrTextXY(x,y+dy*iy,"(no native life)",ColorPI,ColorBG);
    iy++;
    iy++;
    iy++;
    iy++;
  } // no natives
  } // full,target,bio,unowned
  else
  {
  iy++;
  iy++;
  iy++;
  iy++;
  iy++;
  } // end of natives
   iy++;


  // colonists
    if ( ((LocalPlanets[id]&PLANETS_INFO_FULL  )==PLANETS_INFO_FULL  ) ||
         ((LocalPlanets[id]&PLANETS_INFO_TARGET)==PLANETS_INFO_TARGET)
         )
    {
      if (PlanetCargo(id,COLONISTS)!=0)
        {
          GrTextXY(x,y+dy*iy,"Colonists:",ColorPI,ColorBG);
          iy++;
          sprintf(buf,"Pop. : %ld",PlanetCargo(id,COLONISTS));
          GrTextXY(x+dx,y+dy*iy,buf,ColorPI,ColorBG);
        }
      else
        {
          GrTextXY(x,y+dy*iy,"Colonists: None",ColorPI,ColorBG);
          iy++;
          GrTextXY(x,y+dy*iy," (Happy 100%)",ColorPI,ColorBG);
        }
          iy++;
    if ((LocalPlanets[id]&PLANETS_INFO_FULL  )==PLANETS_INFO_FULL  )
      {
        if (PlanetColHappyChange(id)<0) C=ColorPRed;
        else                            C=ColorPI;
        if (PlanetColTax(id)>0)  sprintf(buf,"Taxes: %d%%",PlanetColTax(id));
        else                     sprintf(buf,"Taxes: No");
        GrTextXY(x+dx,y+dy*iy,buf,C      ,ColorBG);
        iy++;
        nathc=PlanetColHappy(id);
        sprintf(buf,"Happy: %d%%",nathc);
        if (nathc<40) C=ColorPRed;
        if (nathc>=40) C=ColorPWarning;
        if (nathc>=70) C=ColorPI;
        GrTextXY(x+dx,y+dy*iy,buf,C    ,ColorBG);
        nathc=PlanetColHappyChange(id);
        if (nathc!=0)
        {
        if (nathc>0)
         {
           sprintf(buf1,"[%+d]",nathc);
           sprintf(buf,"%-5s",buf1);
           GrTextXY(x+dx*14,y+dy*iy,buf,ColorPI,ColorBG);
         }
         else
         {
           sprintf(buf1,"[%+d]",nathc);
           sprintf(buf,"%-5s",buf1);
           GrTextXY(x+dx*14,y+dy*iy,buf,ColorPRed,ColorBG);
         }
         } // nathc!=0
      } // colonist full info
      else
       iy++;
    } // full, target
    else // no full, no target
    if (
        ((LocalPlanets[id]&PLANETS_INFO_UNOWNED)==PLANETS_INFO_UNOWNED) ||
        ((LocalPlanets[id]&PLANETS_INFO_EXPLORE)==PLANETS_INFO_EXPLORE)
       )
      {
      if (PlanetCargo(id,COLONISTS)==0)
        {
        iy++;
        iy++;
        GrTextXY(x,y+dy*iy,"Colonists: None",ColorPI,ColorBG);
        iy++;
        GrTextXY(x,y+dy*iy," (Happy 100%)",ColorPI,ColorBG);
        } // colonist == 0
      else
        {
        GrTextXY(x,y+dy*iy,"Colonists:",ColorPI,ColorBG);
        iy++;
        sprintf(buf,"Pop. : %ld",PlanetCargo(id,COLONISTS));
        GrTextXY(x+dx,y+dy*iy,buf,ColorPI,ColorBG);
        iy++;
        iy++;
        } // colonist != 0
      } // explore, unowned
    else // no unowned, no target, no full, no explore
     {
       iy++;
       iy++;
       iy++;
     }
//    }
//    } //colonist
    iy++;

    iy++;

  // mines,fac,def - full,super spy
  if (((LocalPlanets[id]&PLANETS_INFO_FULL)==PLANETS_INFO_FULL)||
      ((LocalPlanets[id]&PLANETS_INFO_SPY) ==PLANETS_INFO_SPY )  )
    {
    t1=PlanetMines(id);
    t2=PlanetMaxMines(id);
    sprintf(buf,"Mines    : %3d/%-d",t1,t2);
    GrTextXY(x,y+dy*iy,buf,ColorPI,ColorBG);
    if (t1>t2)
    {
     sprintf(buf,"%3d",t1);
     GrTextXY(x+dx*11,y+dy*iy,buf,ColorPRed,ColorBG);
    }

    iy++;
    t1=PlanetFactories(id);
    t2=PlanetMaxFactories(id);
    sprintf(buf,"Factories: %3d/%-d",t1,t2);
    GrTextXY(x,y+dy*iy,buf,ColorPI,ColorBG);
    if (t1>t2)
    {
     sprintf(buf,"%3d",t1);
     GrTextXY(x+dx*11,y+dy*iy,buf,ColorPRed,ColorBG);
    }

    iy++;
    t1=PlanetDefense(id);
    t2=PlanetMaxDefense(id);
    sprintf(buf,"Defense  : %3d/%-d",t1,t2);
    GrTextXY(x,y+dy*iy,buf,ColorPI,ColorBG);
    if (t1>t2)
    {
     sprintf(buf,"%3d",t1);
     GrTextXY(x+dx*11,y+dy*iy,buf,ColorPRed,ColorBG);
    }
    } // fac,mines,def full, sspy
    else  // no full, no sspy
    {
    iy++;
    iy++;
    }  // fac, mines, def
    iy++;
    iy++;

  if (((LocalPlanets[id]&PLANETS_INFO_FULL)   ==PLANETS_INFO_FULL)||
      ((LocalPlanets[id]&PLANETS_INFO_SPY)    ==PLANETS_INFO_SPY )||
      ((LocalPlanets[id]&PLANETS_INFO_TARGET) ==PLANETS_INFO_TARGET ) )
    {
    sprintf(buf,"Supplies: %ld",PlanetCargo(id,SUPPLIES));
    GrTextXY(x,y+dy*iy,buf,ColorPI,ColorBG);
    }
     iy++;


  if (((LocalPlanets[id]&PLANETS_INFO_FULL)   ==PLANETS_INFO_FULL)||
      ((LocalPlanets[id]&PLANETS_INFO_SPY)    ==PLANETS_INFO_SPY )||
      ((LocalPlanets[id]&PLANETS_INFO_TARGET) ==PLANETS_INFO_TARGET )||
      ((LocalPlanets[id]&PLANETS_INFO_DARK)   ==PLANETS_INFO_DARK )  )
    {
    sprintf(buf,"Funds   : %ld",PlanetCargo(id,CREDITS));
    GrTextXY(x,y+dy*iy,buf,ColorPI,ColorBG);
    }
    iy++;

  if ((LocalPlanets[id]&PLANETS_INFO_FULL)   ==PLANETS_INFO_FULL)
    {
    nathc=PlanetColIncome(id)+PlanetNatIncome(id);
    if (nathc!=0)
    {
    sprintf(buf,"Income  : %d",nathc);
    GrTextXY(x,y+dy*iy,buf,ColorPI,ColorBG);
    }
    iy++;
    nathc=PlanetSuppIncome(id);
    if (nathc!=0)
    {
    sprintf(buf,"Sup prod: %d",nathc);
    GrTextXY(x,y+dy*iy,buf,ColorPI,ColorBG);
    }
    } // income only in full
    else
    if ( (LocalPlanets[id]&PLANETS_INFO_UNOWNED)==PLANETS_INFO_UNOWNED)
    {
      iy++;
      nathc=0;
      if (PlanetNatType(id)==Bovinoid)
        {
           nathc= (Uns16)(PlanetNativePopulation(id) / 10000.0);
           nathc= (nathc * gPconfigInfo->ProductionRate[race] / 100.0);
        }
        if (nathc!=0)
        {
        sprintf(buf,"Sup prod: %d",nathc);
        GrTextXY(x,y+dy*iy,buf,ColorPWarning,ColorBG);
        }
    }
    else // no full, no unowned
    if ( (LocalPlanets[id]&PLANETS_INFO_SPY)==PLANETS_INFO_SPY)
    {
        iy++;
        nathc = (PlanetFactories(id) * gPconfigInfo->ProductionRate[PlanetOwner(id)] / 100.0);
        if (nathc!=0)
        {
        sprintf(buf,"Sup prod: %d",nathc);
        GrTextXY(x,y+dy*iy,buf,ColorPWarning,ColorBG);
        }
    }
    else  // all the rest
    iy++;


    iy++;
    iy++;

  if (
      ((LocalPlanets[id]&PLANETS_INFO_FULL)   ==PLANETS_INFO_FULL      )||
      ((LocalPlanets[id]&PLANETS_INFO_UNOWNED)   ==PLANETS_INFO_UNOWNED)
     )
    {
    GrTextXY(x,y+dy*iy,"Minerals:",ColorPI,ColorBG);
    iy++;
    GrTextXY(x,y+dy*iy,"  Surf Core Extr",ColorPI,ColorBG);
    iy++;

    miner=NEUTRONIUM;
    sprintf(buf,"Ne%4ld%5ld    /",PlanetCargo(id,miner),  PlanetCore(id,miner));
    GrTextXY(x,y+dy*iy,buf,ColorPI,ColorBG);
    sprintf(buf,"%4d",PlanetMining(id,miner));
    if (PlanetMining(id,miner)>PlanetCore(id,miner))
      GrTextXY(x+dx*11,y+dy*iy,buf,ColorPRed,ColorBG);
    else
      GrTextXY(x+dx*11,y+dy*iy,buf,ColorPI,ColorBG);
    GrTextXY(x+dx*16,y+dy*iy,ShowInt2(PlanetDensity(id,miner)),ColorPI,ColorBG);
    iy++;

    miner=TRITANIUM;
    sprintf(buf,"Tr%4ld%5ld    /",PlanetCargo(id,miner),  PlanetCore(id,miner));
    GrTextXY(x,y+dy*iy,buf,ColorPI,ColorBG);
    sprintf(buf,"%4d",PlanetMining(id,miner));
    if (PlanetMining(id,miner)>PlanetCore(id,miner))
      GrTextXY(x+dx*11,y+dy*iy,buf,ColorPRed,ColorBG);
    else
      GrTextXY(x+dx*11,y+dy*iy,buf,ColorPI,ColorBG);
    GrTextXY(x+dx*16,y+dy*iy,ShowInt2(PlanetDensity(id,miner)),ColorPI,ColorBG);
    iy++;

    miner=DURANIUM;
    sprintf(buf,"Du%4ld%5ld    /",PlanetCargo(id,miner),  PlanetCore(id,miner));
    GrTextXY(x,y+dy*iy,buf,ColorPI,ColorBG);
    sprintf(buf,"%4d",PlanetMining(id,miner));
    if (PlanetMining(id,miner)>PlanetCore(id,miner))
      GrTextXY(x+dx*11,y+dy*iy,buf,ColorPRed,ColorBG);
    else
      GrTextXY(x+dx*11,y+dy*iy,buf,ColorPI,ColorBG);
    GrTextXY(x+dx*16,y+dy*iy,ShowInt2(PlanetDensity(id,miner)),ColorPI,ColorBG);
    iy++;

    miner=MOLYBDENIUM;
    sprintf(buf,"Mo%4ld%5ld    /",PlanetCargo(id,miner),  PlanetCore(id,miner));
    GrTextXY(x,y+dy*iy,buf,ColorPI,ColorBG);
    sprintf(buf,"%4d",PlanetMining(id,miner));
    if (PlanetMining(id,miner)>PlanetCore(id,miner))
      GrTextXY(x+dx*11,y+dy*iy,buf,ColorPRed,ColorBG);
    else
      GrTextXY(x+dx*11,y+dy*iy,buf,ColorPI,ColorBG);

    GrTextXY(x+dx*16,y+dy*iy,ShowInt2(PlanetDensity(id,miner)),ColorPI,ColorBG);
    iy++;
    }  // minerals full
    else
  if (((LocalPlanets[id]&PLANETS_INFO_SPY)    ==PLANETS_INFO_SPY )||
      ((LocalPlanets[id]&PLANETS_INFO_TARGET) ==PLANETS_INFO_TARGET )||
      ((LocalPlanets[id]&PLANETS_INFO_DARK)   ==PLANETS_INFO_DARK )  )
    {
    GrTextXY(x,y+dy*iy,"Minerals:",ColorPI,ColorBG);
    iy++;
    if ((LocalPlanets[id]&PLANETS_INFO_TARGET) ==PLANETS_INFO_TARGET)
        GrTextXY(x,y+dy*iy,"  Surf",ColorPI,ColorBG);
    else
        GrTextXY(x,y+dy*iy,"  Surf+Core",ColorPI,ColorBG);
    iy++;

    sprintf(buf,"Ne%4ld",PlanetCargo(id,NEUTRONIUM));
    GrTextXY(x,y+dy*iy,buf,ColorPI,ColorBG);
    iy++;
    sprintf(buf,"Tr%4ld",PlanetCargo(id,TRITANIUM));
    GrTextXY(x,y+dy*iy,buf,ColorPI,ColorBG);
    iy++;
    sprintf(buf,"Du%4ld",PlanetCargo(id,DURANIUM));
    GrTextXY(x,y+dy*iy,buf,ColorPI,ColorBG);
    iy++;
    sprintf(buf,"Mo%4ld",PlanetCargo(id,MOLYBDENIUM));
    GrTextXY(x,y+dy*iy,buf,ColorPI,ColorBG);
    iy++;
    }
  else // minerals no full, no ss, no dark,  no target
   {
    iy++;
    iy++;
    iy++;
    iy++;
    iy++;
    iy++;
   }

  iy++;

  if ((LocalPlanets[id]&PLANETS_INFO_UNOWNED)!=PLANETS_INFO_UNOWNED )
  {
  if ((PlanetOwner(id)!=race)&&(!PlayerAllowsAlly(race,PlanetOwner(id),ALLY_PLANETS)) )
  {
  if ( (LocalBases[id]&BASES_INFO_EXIST)==BASES_INFO_EXIST )
       GrTextXY(x,y+dy*iy,"            <Base>",ColorPI,ColorBG);
    if ( (LocalBases[id]&BASES_INFO_NO_BASE)==BASES_INFO_NO_BASE )
       GrTextXY(x,y+dy*iy,"         <No Base>",ColorPI,ColorBG);
  } // owner = race or ally
  else
  if ( (LocalBases[id]&BASES_INFO_EXIST)==BASES_INFO_EXIST )
       GrTextXY(x,y+dy*iy,"            <Base>",ColorPI,ColorBG);

  } // colonist !=0

  GrResetClipBox();
  return True;
}

static char *PlanetClimate(int pID)
{
   int Temp;
   static char Climate[40];

   strcpy(Climate,"");

   if (!IsPlanetExist(pID)) return Climate;

   Temp = PlanetTemp(pID);

   if ((Temp>= 0)&&(Temp<=15))  strcpy(Climate,"Desert");
   if ((Temp>=16)&&(Temp<=35))  strcpy(Climate,"Tropical");
   if ((Temp>=36)&&(Temp<=60))  strcpy(Climate,"Warm");
   if ((Temp>=61)&&(Temp<=85))  strcpy(Climate,"Cool");
   if ((Temp>=86)&&(Temp<=100)) strcpy(Climate,"Arctic");

   return Climate;
}

static char *PlanetNatString(int pID)
{
   Uns16 NatType;
   static char NatString[40];

   strcpy(NatString,"");

   if (!IsPlanetExist(pID)) return NatString;

   NatType = PlanetNatType(pID);

   if (NatType==NoNatives)  strcpy(NatString,"none");
   if (NatType==Humanoid)   strcpy(NatString,"Humanoid");
   if (NatType==Bovinoid)   strcpy(NatString,"Bovinoid");
   if (NatType==Reptilian)  strcpy(NatString,"Reptilian");
   if (NatType==Avian)      strcpy(NatString,"Avian");
   if (NatType==Amorphous)  strcpy(NatString,"Amorphous");
   if (NatType==Insectoid)  strcpy(NatString,"Insectoid");
   if (NatType==Amphibian)  strcpy(NatString,"Amphibian");
   if (NatType==Ghipsoldal) strcpy(NatString,"Ghipsoldal");
   if (NatType==Siliconoid) strcpy(NatString,"Siliconoid");

   return NatString;
}

static char *PlanetNatGovmString(int pID)
{
   Uns16 NatType;
   static char NatString[40];

   strcpy(NatString,"");

   if (!IsPlanetExist(pID)) return NatString;

   NatType = PlanetNatGovm(pID);

   if (NatType==NoGovm)          strcpy(NatString,"NoGovm");
   if (NatType==Anarchy)         strcpy(NatString,"Anarchy");
   if (NatType==PreTribal)       strcpy(NatString,"PreTribal");
   if (NatType==EarlyTribal)     strcpy(NatString,"EarlyTribal");
   if (NatType==Tribal)          strcpy(NatString,"Tribal");
   if (NatType==Feudal)          strcpy(NatString,"Feudal");
   if (NatType==Monarchy)        strcpy(NatString,"Monarchy");
   if (NatType==Representative)  strcpy(NatString,"Representative");
   if (NatType==Participatory)   strcpy(NatString,"Participatory");
   if (NatType==Unity)           strcpy(NatString,"Unity");

   return NatString;
}

Uns16 PlanetMaxFactories(Uns16 pID)
{
  Uns16 ColClans,Max;

  ColClans = PlanetCargo(pID,COLONISTS)/100;
  if (ColClans<100) Max=ColClans;
  else             Max=RND(100.0+sqrt((double)ColClans-100.0));

  return Max;
}

Uns16 PlanetMaxDefense(Uns16 pID)
{
  Uns16 ColClans,Max;

  ColClans = PlanetCargo(pID,COLONISTS)/100;
  if (ColClans<50) Max=ColClans;
  else             Max=RND(50.0+sqrt((double)ColClans-50.0));

  return Max;
}

Uns16 PlanetMaxMines(Uns16 pID)
{
  Uns16 ColClans,Max;

  ColClans = PlanetCargo(pID,COLONISTS)/100;
  if (ColClans<200) Max=ColClans;
  else             Max=RND(200.0+sqrt((double)ColClans-200.0));

  return Max;
}

Uns16 PlanetMining(Uns16 pID, Uns16 Mineral)
{

     double MINING_RATE;
     Uns16  m;

     MINING_RATE = (gPconfigInfo->RaceMiningRate[PlanetOwner(pID)] * PlanetDensity(pID,Mineral)) / 100.0;
     if (PlanetNatType(pID)==Reptilian)
        MINING_RATE = MINING_RATE * 2;

     m=MINING_RATE * PlanetMines(pID) / 100;
     return m;
}

Uns16 PlanetGovTaxRate(Uns16 pID)
{
  Uns16 Gov;

  if (!IsPlanetExist(pID)) return 0;

  if (PlanetNatType(pID)==NoNatives) return 0;
  Gov=PlanetNatGovm(pID);

  if (Gov==NoGovm)         return 0;
  if (Gov==Anarchy)        return 20;
  if (Gov==PreTribal)      return 40;
  if (Gov==EarlyTribal)    return 60;
  if (Gov==Tribal)         return 80;
  if (Gov==Feudal)         return 100;
  if (Gov==Monarchy)       return 120;
  if (Gov==Representative) return 140;
  if (Gov==Participatory)  return 160;
  if (Gov==Unity)          return 180;

  return 0;
}

Uns16 PlanetShipsHissing(Uns16 pID)
{
     Uns16 hiss,i;

     hiss=0;
     if (gPconfigInfo->AllowHiss)
      for (i=1;i<=500;i++)
       if (IsShipExist(i))
        if (PlanetLocationX(pID)==ShipLocationX(i))
         if (PlanetLocationY(pID)==ShipLocationY(i))
           if (gPconfigInfo->PlayerRace[ShipOwner(i)]==Gorn)
            if (ShipMission(i)==9)
             hiss++;
     return hiss;
}

int PlanetNatHappyChange(Uns16 pID)
{
     double NHAPPY_change;
     Uns16 hiss;

     NHAPPY_change  = 5.0;
     NHAPPY_change += PlanetNatGovm(pID)/2.0;
     NHAPPY_change -= sqrt(PlanetNativePopulation(pID)/1000000.0);
     NHAPPY_change -= (PlanetMines(pID)+PlanetFactories(pID))/200.0;
     NHAPPY_change -= PlanetNatTax(pID)*0.85;

     hiss = (int)min(PlanetShipsHissing(pID),gPconfigInfo->MaxShipsHissing);
     NHAPPY_change+= hiss*gPconfigInfo->HissEffectRate;

     if (PlanetNatType(pID)==Avian) NHAPPY_change = NHAPPY_change + 10;

     return (int)NHAPPY_change;
}

int PlanetColHappyChange(Uns16 pID)
{
     double CHAPPY_change;
     Uns16 hiss;
     Uns16 TargetTemp,TempDivisor;

     if ((gPconfigInfo->PlayerRace[PlanetOwner(pID)]==Crystals)&&
          (gPconfigInfo->CrystalsPreferDeserts))
         {
          TargetTemp = 100;
          TempDivisor = 66;
         }
     else
         {
          TargetTemp = 50;
          TempDivisor =33;
         }

     CHAPPY_change = 10.0;
     CHAPPY_change -= sqrt(PlanetCargo(pID,COLONISTS)/1000000.0);
     CHAPPY_change -= fabs(PlanetTemp(pID)-TargetTemp)/TempDivisor;
     CHAPPY_change -= (PlanetMines(pID)+PlanetFactories(pID))/300.0;
     CHAPPY_change -= PlanetColTax(pID)*0.8;

     hiss = (int)min(PlanetShipsHissing(pID),gPconfigInfo->MaxShipsHissing);
     CHAPPY_change+= hiss*gPconfigInfo->HissEffectRate;

     return (int)CHAPPY_change;
}

Uns16 PlanetColIncome(Uns16 pID)
{
     double MC;

     if (PlanetColHappy(pID)<30) return 0;
     MC= floor(((double)PlanetCargo(pID,COLONISTS)/100.0 * PlanetColTax(pID) * 5.0 + 2500.0) / 5000.0);
     return floor((MC * gPconfigInfo->ColonistTaxRate[PlanetOwner(pID)] + 50.0) / 100.0);
}

Uns16 PlanetNatIncome(Uns16 pID)
{
     double MC;

     if (PlanetNatHappy(pID)<30) return 0;
     if (PlanetNatType(pID)==Amorphous) return 0;

     MC  = min(floor(((double)PlanetNativePopulation(pID)/100.0 * PlanetNatGovm(pID) * PlanetNatTax(pID) + 2500.0) / 5000.0)
               ,PlanetCargo(pID,COLONISTS)/100.0);
     if (PlanetNatType(pID)==Insectoid) MC = MC * 2;

    return floor((MC * gPconfigInfo->NativeTaxRate[PlanetOwner(pID)] + 50.0) / 100.0);
}

Uns16 PlanetSuppIncome(Uns16 pID)
{
     Uns16 S;

     S = PlanetFactories(pID);
     if (PlanetNatType(pID)==Bovinoid) S+= (Uns16)min(PlanetNativePopulation(pID) / 10000.0, PlanetCargo(pID,COLONISTS)/100.0);

     return floor(S * gPconfigInfo->ProductionRate[PlanetOwner(pID)] / 100.0);
}

void IntegrateXYMaps(void)
{
 Uns16 i,j;
 Uns16 Map[500][3], LoadedMap[500][3];
 char buf[20];
 FILE *f;
 Boolean save;

 save =False;
 for(j=0;j<500;j++)
 {
     Map[j][0]=0;
     Map[j][1]=0;
     Map[j][2]=0;
 }

 for (i=1;i<12;i++)
 {
   sprintf(buf,"xyplan%d.dat",i);
   f=OpenInputFile(buf,GAME_DIR_ONLY|NO_MISSING_ERROR);
   if (f!=NULL)
   {
     fread(LoadedMap,sizeof(LoadedMap),1,f);
     if (f!=NULL) fclose(f);
     save=True;
     for(j=0;j<500;j++)
     {
       if ((LoadedMap[j][0]>500)&&(LoadedMap[j][0]<3500))
       {
       Map[j][0]=LoadedMap[j][0];
       Map[j][1]=LoadedMap[j][1];
       Map[j][2]=0;
       }
       if ((LoadedMap[j][1]>500)&&(LoadedMap[j][1]<3500))
       {
       Map[j][0]=LoadedMap[j][0];
       Map[j][1]=LoadedMap[j][1];
       Map[j][2]=0;
       }
     } // check for stars moved out
   }
 }

 if (save)
 {
   f=OpenOutputFile("xyplan.dat",GAME_DIR_ONLY);
   fwrite(Map,sizeof(Map),1,f);
   if (f!=NULL) fclose(f);
 }
 return;
}

static char *SensorSweepString(char SensorType)
{
   static char SensorString[40];

   if (SensorType==SS_NO_INFO)     strcpy(SensorString,"No Info");
   if (SensorType==SS_MINIMAL)     strcpy(SensorString,"Minimal");
   if (SensorType==SS_LIGHT)       strcpy(SensorString,"Light");
   if (SensorType==SS_MODERATE)    strcpy(SensorString,"Moderate");
   if (SensorType==SS_SUBSTANCIAL) strcpy(SensorString,"Substantial");
   if (SensorType==SS_HEAVY)       strcpy(SensorString,"Heavy");
   if (SensorType==SS_VERY_HEAVY)  strcpy(SensorString,"Very Heavy");

   return SensorString;
}

void LoadUtilFile(int tmprace)
{
 FILE *f;
 char s[80],buf[2000];
 int i, MineID;
 Boolean Found;

 struct record_head         head;
 struct util_exploration    explore;
 struct util_super_spy      super_spy;
 struct util_dark_sense     dark_sense;
 struct util_planet_target  planet_target;
 struct util_base_target    base_target;
 struct util_sensor_sweep   sensor_sweep;
 struct util_mine_scan      mine_scan;
// struct util_wormhole_scan  wormhole_scan;
// struct util_control        control;
 struct util_bioscan        bioscan;

 sprintf(s,"util%d.dat",tmprace);
 f=OpenInputFile(s,GAME_DIR_ONLY|NO_MISSING_ERROR);
 if (f==NULL) return;
/* same information about explosion are sent to every one
   so we can zero counter each time
*/
 explosion_number=0;

 while (!feof(f))
 {
  fread(&head,sizeof(head),1,f);
  if (!feof(f))
   {
// explore record
   if (head.recordtype==5)
   {
     fread(&explore.PlanetID    ,sizeof(explore.PlanetID)   ,1,f);
     fread(&explore.Temperature ,sizeof(explore.Temperature),1,f);
     fread(&explore.Owner       ,sizeof(explore.Owner)      ,1,f);
     fread(&explore.Colonists   ,sizeof(explore.Colonists)  ,1,f);
     fread(&explore.HasStarbase ,sizeof(explore.HasStarbase),1,f);

     head.recordlen=0;
     if (IsPlanetExist(explore.PlanetID))
     {
     PutPlanetTemp (explore.PlanetID,explore.Temperature);
     PutPlanetOwner(explore.PlanetID,explore.Owner);
     PutPlanetCargo(explore.PlanetID,COLONISTS,explore.Colonists);

     LocalPlanets[explore.PlanetID]=LocalPlanets[explore.PlanetID]|PLANETS_INFO_EXPLORE;
     if (explore.HasStarbase==1)
     LocalBases  [explore.PlanetID]=LocalBases  [explore.PlanetID]|BASES_INFO_EXIST;
     else
     LocalBases  [explore.PlanetID]=LocalBases  [explore.PlanetID]|BASES_INFO_NO_BASE;
     }
   }

// super spy record
   if (head.recordtype==4)
   {
     fread(&super_spy.PlanetID ,sizeof(super_spy.PlanetID) ,1,f);
     fread(&super_spy.Mines    ,sizeof(super_spy.Mines)    ,1,f);
     fread(&super_spy.Factories,sizeof(super_spy.Factories),1,f);
     fread(&super_spy.Defences ,sizeof(super_spy.Defences) ,1,f);
     fread(&super_spy.FriendlyCode,3,1,f);
     super_spy.FriendlyCode[3]=0;
     fread(&super_spy.Neutronium,sizeof(super_spy.Neutronium),1,f);
     fread(&super_spy.Tritanium ,sizeof(super_spy.Tritanium) ,1,f);
     fread(&super_spy.Duranium  ,sizeof(super_spy.Duranium)  ,1,f);
     fread(&super_spy.Molybdenum,sizeof(super_spy.Molybdenum),1,f);
     fread(&super_spy.Credits   ,sizeof(super_spy.Credits)   ,1,f);
     fread(&super_spy.Supplies  ,sizeof(super_spy.Supplies)  ,1,f);

     head.recordlen=0;

     if (IsPlanetExist(super_spy.PlanetID))
     {
     PutPlanetMines    (super_spy.PlanetID,super_spy.Mines     );
     PutPlanetFactories(super_spy.PlanetID,super_spy.Factories );
     PutPlanetDefense  (super_spy.PlanetID,super_spy.Defences  );
     PutPlanetFC       (super_spy.PlanetID,super_spy.FriendlyCode);
     PutPlanetCargo(super_spy.PlanetID,NEUTRONIUM  ,super_spy.Neutronium);
     PutPlanetCargo(super_spy.PlanetID,TRITANIUM   ,super_spy.Tritanium);
     PutPlanetCargo(super_spy.PlanetID,DURANIUM    ,super_spy.Duranium);
     PutPlanetCargo(super_spy.PlanetID,MOLYBDENIUM ,super_spy.Molybdenum);
     PutPlanetCargo(super_spy.PlanetID,CREDITS     ,super_spy.Credits);
     PutPlanetCargo(super_spy.PlanetID,SUPPLIES    ,super_spy.Supplies);

     LocalPlanets[super_spy.PlanetID]=LocalPlanets[super_spy.PlanetID]|PLANETS_INFO_SPY;
     }
   }

// dark sense record
   if (head.recordtype==3)
   {
     fread(&dark_sense.PlanetID    ,sizeof(dark_sense.PlanetID)   ,1,f);
     fread(&dark_sense.Owner       ,sizeof(dark_sense.Owner)      ,1,f);
     fread(&dark_sense.Neutronium  ,sizeof(dark_sense.Neutronium) ,1,f);
     fread(&dark_sense.Tritanium   ,sizeof(dark_sense.Tritanium)  ,1,f);
     fread(&dark_sense.Duranium    ,sizeof(dark_sense.Duranium)   ,1,f);
     fread(&dark_sense.Molybdenum  ,sizeof(dark_sense.Molybdenum) ,1,f);
     fread(&dark_sense.Credits     ,sizeof(dark_sense.Credits)    ,1,f);
     fread(&dark_sense.HasStarbase ,sizeof(dark_sense.HasStarbase),1,f);

     head.recordlen=0;

       if (IsPlanetExist(dark_sense.PlanetID))
       {
     PutPlanetOwner    (dark_sense.PlanetID,dark_sense.Owner);
     PutPlanetCargo(dark_sense.PlanetID,NEUTRONIUM  ,dark_sense.Neutronium);
     PutPlanetCargo(dark_sense.PlanetID,TRITANIUM   ,dark_sense.Tritanium);
     PutPlanetCargo(dark_sense.PlanetID,DURANIUM    ,dark_sense.Duranium);
     PutPlanetCargo(dark_sense.PlanetID,MOLYBDENIUM ,dark_sense.Molybdenum);
     PutPlanetCargo(dark_sense.PlanetID,CREDITS     ,dark_sense.Credits);

     LocalPlanets[dark_sense.PlanetID]=LocalPlanets[dark_sense.PlanetID]|PLANETS_INFO_DARK;
     if (dark_sense.HasStarbase==1)
     LocalBases  [dark_sense.PlanetID]=LocalBases  [dark_sense.PlanetID]|BASES_INFO_EXIST;
     else
     LocalBases  [dark_sense.PlanetID]=LocalBases  [dark_sense.PlanetID]|BASES_INFO_NO_BASE;
     }
   }
// base target data record
   if (head.recordtype==11)
   {
     fread(&base_target.BaseID    ,sizeof(base_target.BaseID)   ,1,f);
     fread(&base_target.BaseOwner ,sizeof(base_target.BaseOwner),1,f);

     head.recordlen=0;

     LocalBases[base_target.BaseID]=LocalBases[base_target.BaseID]|BASES_INFO_EXIST;
   }


// planet target data record
   if (head.recordtype==12)
   {
     fread(&planet_target.PlanetID         ,sizeof(planet_target.PlanetID)   ,1,f);
     fread(&planet_target.PlanetOwner      ,sizeof(planet_target.PlanetOwner),1,f);
     fread(&planet_target.Temperature      ,sizeof(planet_target.Temperature),1,f);
     fread(&planet_target.NativesType      ,sizeof(planet_target.NativesType),1,f);
     fread(&planet_target.NativeGovernment ,sizeof(planet_target.NativeGovernment) ,1,f);
     fread(&planet_target.NumNatives       ,sizeof(planet_target.NumNatives) ,1,f);
     fread(&planet_target.Neutronium       ,sizeof(planet_target.Neutronium) ,1,f);
     fread(&planet_target.Tritanium        ,sizeof(planet_target.Tritanium)  ,1,f);
     fread(&planet_target.Duranium         ,sizeof(planet_target.Duranium)   ,1,f);
     fread(&planet_target.Molybdenum       ,sizeof(planet_target.Molybdenum) ,1,f);
     fread(&planet_target.NumColonists     ,sizeof(planet_target.NumColonists),1,f);
     fread(&planet_target.Supplies         ,sizeof(planet_target.Supplies)   ,1,f);
     fread(&planet_target.Credits          ,sizeof(planet_target.Credits)    ,1,f);

     head.recordlen=0;

       if (IsPlanetExist(planet_target.PlanetID))
       {
     PutPlanetTemp   (planet_target.PlanetID ,planet_target.Temperature);
     PutPlanetOwner  (planet_target.PlanetID ,planet_target.PlanetOwner);
     PutPlanetNatType(planet_target.PlanetID ,planet_target.NativesType);
     PutPlanetNatGovm(planet_target.PlanetID ,planet_target.NativeGovernment);
     PutPlanetNativePopulation (planet_target.PlanetID ,planet_target.NumNatives);
     PutPlanetCargo(planet_target.PlanetID,NEUTRONIUM  ,planet_target.Neutronium);
     PutPlanetCargo(planet_target.PlanetID,TRITANIUM   ,planet_target.Tritanium);
     PutPlanetCargo(planet_target.PlanetID,DURANIUM    ,planet_target.Duranium);
     PutPlanetCargo(planet_target.PlanetID,MOLYBDENIUM ,planet_target.Molybdenum);
     PutPlanetCargo(planet_target.PlanetID,COLONISTS   ,planet_target.NumColonists);
     PutPlanetCargo(planet_target.PlanetID,SUPPLIES    ,planet_target.Supplies);
     PutPlanetCargo(planet_target.PlanetID,CREDITS     ,planet_target.Credits);

     LocalPlanets[planet_target.PlanetID]=LocalPlanets[planet_target.PlanetID]|PLANETS_INFO_TARGET;
     }
   }

// mine scan record

   if (head.recordtype==0)
   {
       fread(&mine_scan.MinefieldID     ,sizeof(mine_scan.MinefieldID )    ,1,f);
       fread(&mine_scan.XLocation       ,sizeof(mine_scan.XLocation )      ,1,f);
       fread(&mine_scan.YLocation       ,sizeof(mine_scan.YLocation )      ,1,f);
       fread(&mine_scan.Owner           ,sizeof(mine_scan.Owner )          ,1,f);
       fread(&mine_scan.MinefieldUnits  ,sizeof(mine_scan.MinefieldUnits ) ,1,f);
       fread(&mine_scan.IsWeb           ,sizeof(mine_scan.IsWeb )          ,1,f);
       fread(&mine_scan.FCodePlanet     ,sizeof(mine_scan.FCodePlanet )    ,1,f);
       fread(&mine_scan.ScanType        ,sizeof(mine_scan.ScanType )       ,1,f);

       head.recordlen=0;

     /* Delete minefield in order to put loaded data */
       PutMinefieldUnits(mine_scan.MinefieldID,0);

       if (mine_scan.ScanType==0) LocalMinefields[mine_scan.MinefieldID]=MINEFIELDS_INFO_FULL;
       if (mine_scan.ScanType==2) LocalMinefields[mine_scan.MinefieldID]=MINEFIELDS_INFO_FULL;
       if (mine_scan.ScanType==1)
       {
       LocalMinefields[mine_scan.MinefieldID]=MINEFIELDS_INFO_SWEPT_OUT;
       mine_scan.MinefieldUnits=4;
       }

       MineID = CreateMinefield(mine_scan.XLocation,mine_scan.YLocation,
                                mine_scan.Owner,mine_scan.MinefieldUnits,mine_scan.IsWeb);
       MinefieldPlanetID[mine_scan.MinefieldID]=mine_scan.FCodePlanet;
    /*  check if minefield is created good */
       if (MineID!=mine_scan.MinefieldID) Info("Error creating minefield ID:%d",mine_scan.MinefieldID);
//       Info("%d %d %d",mine_scan.MinefieldID,mine_scan.Owner,(Uns16)sqrt(mine_scan.MinefieldUnits));
   }

//  sensor sweep record
   if (head.recordtype==6)
   {
       fread(&sensor_sweep.PlanetID  ,sizeof(sensor_sweep.PlanetID )  ,1,f);
       fread(&sensor_sweep.Owner     ,sizeof(sensor_sweep.Owner  )    ,1,f);
       fread(&sensor_sweep.Activity  ,sizeof(sensor_sweep.Activity )  ,1,f);

       head.recordlen=0;

       if (IsPlanetExist(sensor_sweep.PlanetID))
       {
       PutPlanetOwner(sensor_sweep.PlanetID,sensor_sweep.Owner);
       SensorSweepActivity[sensor_sweep.PlanetID] = (char) sensor_sweep.Activity;

       LocalPlanets[sensor_sweep.PlanetID]=LocalPlanets[sensor_sweep.PlanetID]|PLANETS_INFO_SENSOR;
       }
   }

// bio scan record
   if (head.recordtype==23)
   {
       fread(&bioscan.PlanetID        ,sizeof(bioscan.PlanetID        ),1,f);
       fread(&bioscan.NativesType     ,sizeof(bioscan.NativesType     ),1,f);
       fread(&bioscan.NativePopulation,sizeof(bioscan.NativePopulation),1,f);
       fread(&bioscan.Temperature     ,sizeof(bioscan.Temperature     ),1,f);

       head.recordlen=0;

       if (IsPlanetExist(bioscan.PlanetID))
       {
         PutPlanetTemp   (bioscan.PlanetID,bioscan.Temperature);
         PutPlanetNatType(bioscan.PlanetID,bioscan.NativesType);
         PutPlanetNativePopulation (bioscan.PlanetID ,bioscan.NativePopulation);

         LocalPlanets[bioscan.PlanetID]=LocalPlanets[bioscan.PlanetID]|PLANETS_INFO_BIOSCAN;
       }
   }

// battle result
   if (head.recordtype==7)
   {
       fread(&VCRBattleResult[VCRBattleResultNumber],30,1,f);
       /* check if this info is allready read */
       Found=False;
       for (i=0;i<VCRBattleResultNumber;i++)
       if ((VCRBattleResult[i].LeftID==VCRBattleResult[VCRBattleResultNumber].LeftID)&&
           (VCRBattleResult[i].RightID==VCRBattleResult[VCRBattleResultNumber].RightID)&&
           (VCRBattleResult[i].IsPlanet==VCRBattleResult[VCRBattleResultNumber].IsPlanet))
               Found=True;

       if (!Found) VCRBattleResultNumber++;
       head.recordlen=0;
   }

// explosions
   if (head.recordtype==1)
   {
       fread(&explosion[explosion_number],6,1,f);
       explosion_number++;
       head.recordlen=0;
   }

   i=head.recordlen;
   while (i>0)
   {
    if (i>2000)   fread(buf,2000,1,f);
    else          fread(buf,i,1,f);
    i-=2000;
   }
  } // if feof
 } // while feof
 if (f!=NULL) fclose(f);
}

Boolean CheckUtilFile(int race)
{
 FILE *f;
 char s[80];

 struct record_head         head;
 struct util_control        control;

 sprintf(s,"util%d.dat",race);
 f=OpenInputFile(s,GAME_DIR_ONLY|NO_MISSING_ERROR);
 if (f==NULL) return False;

   fread(&head,sizeof(head),1,f);
   // record 13 should be first if not exit
   if (head.recordtype==13)
   { // read control record
       fread(&control.HostRunTime         ,sizeof(control.HostRunTime),1,f);
       fread(&control.TurnNumber          ,sizeof(control.TurnNumber),1,f);
       fread(&control.Player              ,sizeof(control.Player),1,f);
       fread(&control.PHOSTVersionMajor   ,sizeof(control.PHOSTVersionMajor),1,f);
       fread(&control.PHOSTVersionMinor   ,sizeof(control.PHOSTVersionMinor),1,f);
       fread(&control.HullspecDigest      ,sizeof(control.HullspecDigest),1,f);
       fread(&control.EngspecDigest       ,sizeof(control.EngspecDigest),1,f);
       fread(&control.BeamspecDigest      ,sizeof(control.BeamspecDigest),1,f);
       fread(&control.TorpspecDigest      ,sizeof(control.TorpspecDigest),1,f);
       fread(&control.TruehullDigest      ,sizeof(control.TruehullDigest),1,f);
       fread(&control.PlanetXYDigest      ,sizeof(control.PlanetXYDigest),1,f);
       fread(&control.ConfigDigest        ,sizeof(control.ConfigDigest),1,f);
       fread(&control.RacenameDigest      ,sizeof(control.RacenameDigest),1,f);
       fread(&control.GameName            ,sizeof(control.GameName),1,f);
       fread(&control.PHOSTVersionRelease ,sizeof(control.PHOSTVersionRelease),1,f);
       if (f!=NULL) fclose(f);

       if (control.Player!=race) return False;

       strncpy(s,control.HostRunTime,18);
       s[18]=0;

       if (strcmp(s,TimeSyg)!=0)  return False;
       if (control.TurnNumber!=turnnumber) return False;
       return True;
   }
   else // not record 13 first
   {
    if (f!=NULL) fclose(f);
    return False;
   }
  return False;
}

Boolean FindNearestPlanet(Uns16 mx, Uns16 my, Uns16 *PID, Uns16 *d)
{
  Uns16 i,minp;
  double dd,minpd;

         minp=0;
         minpd=gPconfigInfo->WraparoundRectangle[2]-gPconfigInfo->WraparoundRectangle[0];
         for (i=1;i<=500;i++)
          if (IsPlanetExist(i))
          {
          dd=DistWrap(mx,my,PlanetLocationX(i),PlanetLocationY(i));
          if (dd<minpd)
             { minpd=dd;
                minp=i; }
           }
        *d=(Uns16)minpd;
        *PID=minp;
        if (minp!=0)  return True;
        else          return False;
}

Boolean FindNearestShip(Uns16 mx, Uns16 my, Uns16 *SID, Uns16 *d)
{
  Uns16 i,mins;
  double dd,minsd;

         mins=0;
         minsd=gPconfigInfo->WraparoundRectangle[2]-gPconfigInfo->WraparoundRectangle[0];
         for (i=1;i<=500;i++)
          if (IsShipExist(i))
             {
             dd=DistWrap(mx,my,ShipLocationX(i),ShipLocationY(i));
             if (dd<minsd)
             { minsd=dd;
                mins=i; }
             }
        *d=(Uns16)minsd;
        *SID=mins;
        if (mins!=0)  return True;
        else          return False;

}

Boolean FindNearestMinefield(Uns16 mx, Uns16 my, Uns16 *MID, Uns16 *d)
{
  Uns16 i,minm;
  double dd,minmd;

         minm=0;
         minmd=gPconfigInfo->WraparoundRectangle[2]-gPconfigInfo->WraparoundRectangle[0];
         for (i=1;i<=500;i++)
          if (IsMinefieldExist(i))
             {
             dd=DistWrap(mx,my,MinefieldPositionX(i),MinefieldPositionY(i));
             if (dd<minmd)
             { minmd=dd;
                minm=i; }
             }
        *d=(Uns16)minmd;
        *MID=minm;
        if (minm!=0)  return True;
        else          return False;
}



void BuildShipsAtPos(int X, int Y)
{
  int i;

      for (i=1;i<=500;i++)
          if (IsShipExist(i))
            if (((ShipLocationX(i)-X)==0)&&
                ((ShipLocationY(i)-Y)==0))
           {
           objects_in_pos[objects_in_pos_no].type   =OBJECT_SHIP;
           objects_in_pos[objects_in_pos_no].id     =i;
           objects_in_pos[objects_in_pos_no].hull   =ShipHull(i);
           objects_in_pos[objects_in_pos_no].owner  =ShipOwner(i);
           objects_in_pos[objects_in_pos_no].reowner=ShipRemoteController(i);
           objects_in_pos_no++;
           }

}

void BuildMinesAtPos(int X, int Y)
{
  int i;

      for (i=1;i<=500;i++)
          if (IsMinefieldExist(i))
            if (((MinefieldPositionX(i)-X)==0)&&
                ((MinefieldPositionY(i)-Y)==0))
           {
           objects_in_pos[objects_in_pos_no].type   =OBJECT_MINEFIELD;
           objects_in_pos[objects_in_pos_no].id     =i;
           objects_in_pos[objects_in_pos_no].owner  =MinefieldOwner(i);
           objects_in_pos_no++;
           }
}

Boolean ShowMinefield(int x,int y, int ddx, int ddy, int id)
{
  int dx=FontDX,dy=FontDY,iy=0,i;
  char buf[100];
  char buf1[100];
  GrColor C;
  double units,bn;

  double safe1,p,r,safe2;

  if (!IsMinefieldExist(id)) return False;

  GrSetClipBox(x,y,x+ddx,y+ddy);
  GrFilledBox (x,y,x+ddx,y+ddy,ColorBG);

  sprintf(buf,"%s",RaceNameAdjective(MinefieldOwner(id),NULL));
  GrTextXY(x,y+dy*iy,buf,ColorSt,ColorBG);
  iy++;
  if (IsMinefieldWeb(id))
  sprintf(buf,"Mine WEB %d",id);
  else
  sprintf(buf,"Mine field %d",id);

  GrTextXY(x,y+dy*iy,buf,ColorSt,ColorBG);
  iy++;
  GrTextXY(x,y+dy*iy,"(now)",ColorSt,ColorBG);
  iy++;
  iy++;
  sprintf(buf,"Center:(%4d,%4d)",MinefieldPositionX(id),MinefieldPositionY(id));
  GrTextXY(x,y+dy*iy,buf,ColorSt,ColorBG);
  iy++;
  if (LocalMinefields[id]==MINEFIELDS_INFO_SWEPT_OUT)
  {
  sprintf(buf,"Radius: " );
  GrTextXY(x,y+dy*iy,buf,ColorSt,ColorBG);
  iy++;
  sprintf(buf,"Units : SWEPT OUT");
  GrTextXY(x,y+dy*iy,buf,ColorSt,ColorBG);
  // no more info
  GrResetClipBox();
  return True;
  }
  // other info
  sprintf(buf,"Radius: %d",RND(sqrt(MinefieldUnits(id))) );
  GrTextXY(x,y+dy*iy,buf,ColorSt,ColorBG);
  iy++;
  sprintf(buf,"Units : %ld",MinefieldUnits(id) );
  GrTextXY(x,y+dy*iy,buf,ColorSt,ColorBG);
  iy++;

  /* FCode */
  iy++;
  if (MinefieldPlanetID[id]!=0)
  {
  sprintf(buf,"FCode Planet: %d",MinefieldPlanetID[id]);
  GrTextXY(x,y+dy*iy,buf,ColorSt,ColorBG);
  iy++;

  if ((LocalPlanets[MinefieldPlanetID[id]]&PLANETS_INFO_FULL)    ==PLANETS_INFO_FULL )
  {
  GrTextXY(x,y+dy*iy,"FCode       : ",ColorSt,ColorBG);

  if (IsPlanetFCSpecial(PlanetFCode(MinefieldPlanetID[id],NULL)))
  C=ColorPWarning;
  else
  C=ColorSt;

  sprintf(buf,"%s",PlanetFCode(MinefieldPlanetID[id],NULL));
  GrTextXY(x+dx*14,y+dy*iy,buf,C      ,ColorBG);
  }
  }
  else
  {
  iy++;
  }
  iy++;

  /* No universal MFC now */
  if (UniversalFCPID[MinefieldOwner(id)]!=0)
  {
     sprintf(buf,"UniFC planet : %3d",UniversalFCPID[MinefieldOwner(id)]);
     GrTextXY(x,y+dy*iy,buf,ColorSt,ColorBG);
     iy++;
     sprintf(buf,"Universal FC : %s",UniversalFC[MinefieldOwner(id)]);
     GrTextXY(x,y+dy*iy,buf,ColorSt,ColorBG);
  }
  else
  iy++;

  iy++;
  iy++;

  /* Torps needed */
  sprintf(buf,"Torps rate %d %%",gPconfigInfo->UnitsPerTorpRate[MinefieldOwner(id)]);
  GrTextXY(x,y+dy*iy,buf,ColorSt,ColorBG);
  iy++;
  GrTextXY(x,y+dy*iy,"Torps needed",ColorSt,ColorBG);
  iy++;

  units=(double)MinefieldUnits(id)/(double)gPconfigInfo->UnitsPerTorpRate[MinefieldOwner(id)]*100.0;
  for (i=1;i<=TORP_NR;i++)
  {
     TorpName(i,buf);
     strncpy(buf1,buf,6);
     buf1[6]=0;
     sprintf(buf,"%10.2f %s",units/i/i,buf1);
     GrTextXY(x,y+dy*iy,buf,ColorSt,ColorBG);
     iy++;
  }
  iy++;

  /* Best beams needed to sweep out, (now, no decay)  */
  GrTextXY(x,y+dy*iy,"To sweep use :",ColorSt,ColorBG);
  iy++;

  if (IsMinefieldWeb(id))
    bn=(double)MinefieldUnits(id)/((double)BEAM_NR*(double)BEAM_NR*(double)gPconfigInfo->WebSweepRate[race]);
  else
    bn=(double)MinefieldUnits(id)/((double)BEAM_NR*(double)BEAM_NR*(double)gPconfigInfo->MineSweepRate[race]);

   if ((double)((Uns16)(bn))!=bn) bn=floor(bn)+1.0;
   else bn=floor(bn);

  BeamName(BEAM_NR,buf);
  strncpy(buf1,buf,15);
  buf1[15]=0;
  sprintf(buf,"%d x %s",(Uns16)bn,buf1);
  GrTextXY(x,y+dy*iy,buf,ColorSt,ColorBG);
  iy++;
  iy++;

  /* Probablility to go center with w=9 */

  r=RND(sqrt(MinefieldUnits(id)));
  sprintf(buf,"%3d LY travel prob",(Uns16)r);
  GrTextXY(x,y+dy*iy,buf,ColorSt,ColorBG);
  iy++;

  if (IsMinefieldWeb(id))
    p = (100.0-(double)gPconfigInfo->WebMineHitOdds[MinefieldOwner(id)])/100.0;
  else
    p = (100.0-(double)gPconfigInfo->MineHitOdds[MinefieldOwner(id)])/100.0;

  safe1=exp(r*log(p));

  if (IsMinefieldWeb(id))
    p = (100.0-(double)gPconfigInfo->WebMineHitOdds[MinefieldOwner(id)])/100.0;
  else
    p = (100.0-(double)gPconfigInfo->MineHitOddsWhenCloakedX10[MinefieldOwner(id)]/10)/100.0;

  safe2=exp(r*log(p));

  sprintf(buf,"  %5.2f%% (%5.2f%%)",safe1*100.0,safe2*100.0);
  GrTextXY(x,y+dy*iy,buf,ColorSt,ColorBG);
  iy++;
  iy++;


  /* Units and radius next turn , decay */
  if (IsMinefieldWeb(id))
    units=(double)MinefieldUnits(id)*(100.0-(double)gPconfigInfo->WebDecay)/100.0;
  else
   units=(double)MinefieldUnits(id)*(100.0-(double)gPconfigInfo->MineDecayRate[MinefieldOwner(id)])/100.0 ;

  GrTextXY(x,y+dy*iy,"Next turn",ColorSt,ColorBG);
  iy++;

  sprintf(buf,"Radius: %d",RND(sqrt(units)) );
  GrTextXY(x,y+dy*iy,buf,ColorSt,ColorBG);
  iy++;
  sprintf(buf,"Units : %d",(Uns16)units );
  GrTextXY(x,y+dy*iy,buf,ColorSt,ColorBG);
  iy++;

  GrResetClipBox();
  return True;
}

void MessageReader(int x, int y, int ddx, int ddy)
{

   char buf[80],what;
   Uns16 dy=13,mx=FontDX*4,my=(FontDY+1)*10,len;
   Uns32 lj,j_,j__,y_;
   static Uns16 j=0;
   float mdx;

   GrMouseEvent evt;
   Uns32 c=0;

   GrFilledBox(x,y,x+ddx,y+ddy,ColorBG);

   sprintf(buf,"%3d INCOMING MESSAGES",MessageNumber);
   GrTextXY(x,y+dy*1,buf,ColorSt,ColorBG);

   if (MessageNumber==0) return;

   mdx = (ddx-25.0*FontDX)/MessageNumber;
   if (mdx>10) mdx=10;
   for (j_=0;j_<MessageNumber;j_++)
    GrFilledBox((Uns16)((float)x+22.0*FontDX+j_*mdx),y+FontDY*1.5,(Uns16)((float)x+22.0*FontDX+j_*mdx+1),y+FontDY*1.5+1,ColorMessage);

//   j=0;
   lj=1;
   sprintf(buf,"%d",j+1);
   len=4;
   if (j<1000) len=3;
   if (j<100) len=2;
   if (j<10) len=1;
   GrFilledBox(x+22.0*FontDX-FontDY,y+2*dy,x+22.0*FontDX+ddx-25.0*FontDX+FontDY,y+dy*3,ColorBG);
   GrTextXY((Uns16)((float)x+22.0*FontDX+j*mdx-len*4),y+dy*2,buf,ColorMessNow,ColorBG);
   GrFilledBox((Uns16)((float)x+22.0*FontDX+j*mdx),y+FontDY*1.5,(Uns16)((float)x+22.0*FontDX+j*mdx+1),y+FontDY*1.5+1,ColorMessNow);

/* Message frame */
   GrBox(x+mx-3,y+my-3,x+mx+40*FontDX+3,y+my+dy*20+3,ColorMessage);
   GrBox(x+mx-2,y+my-2,x+mx+40*FontDX+2,y+my+dy*20+2,ColorMessage);
   j__=0;
   y_ =0;
   GrFilledBox(x+mx,y+my,x+mx+40*FontDX,y+my+dy*20,ColorBG);

/* Print message */
     for (j_=0;j_<(MessageIndex+j)->Length;j_++)
     {
       what =*(MessageData+((MessageIndex+j)->Offset-MessageNumber*6-3+j_));
       if (what!='\r') {  buf[j__]=what; j__++; }
       else { buf[j__]=0; j__=0;  GrTextXY(x+mx,y+my+dy*y_,buf,ColorMessage,ColorBG); y_++; }
     }
/* Extract message 1 line */
     for (j_=0;j_<(MessageIndex+j)->Length;j_++)
     {
       what =*(MessageData+((MessageIndex+j)->Offset-MessageNumber*6-3+j_));
       if (what!='\r') buf[j_]=what;
       else { buf[j_]=0;
              break; }
     }
     LocateEventOnMap(buf);
/* Show if going to map aviable */
   GrFilledBox(x+mx,y+my+dy*21,x+mx+40*FontDX,y+my+dy*22,ColorBG);
   if (MessEvent.go_to)
      GrTextXY(x+mx,y+my+dy*21,"Map",ColorMessage,ColorBG);

  while (c!=GrKey_Escape)
  {
  GrMouseGetEventT(GR_M_EVENT,&evt,0L);
  if (evt.flags & (GR_M_KEYPRESS))
  {
     c=evt.key;

     if ((j<MessageNumber-1)&&(c==GrKey_Right)) { lj=j; j++; }
     else
     if ((j==MessageNumber-1)&&(c==GrKey_Right)) { lj=j; j=0; }

     if ((j>0)      &&(c==GrKey_Left )) { lj=j; j--; }
     else
     if ((j==0)      &&(c==GrKey_Left )) { lj=j; j=MessageNumber-1; }

     /* To go to place on map press 'm' or Enter */
     if ((c==GrKey_Return)&&(MessEvent.go_to)) { c=GrKey_Escape; }
     if ((c==GrKey_m     )&&(MessEvent.go_to)) { c=GrKey_Escape; }

     if (lj!=j) // cursor pressed
     {
     j__=0;
     y_ =0;

/* Print message */
     GrFilledBox(x+mx,y+my,x+mx+40*FontDX,y+my+dy*20,ColorBG);
     for (j_=0;j_<(MessageIndex+j)->Length;j_++)
     {
       what =*(MessageData+((MessageIndex+j)->Offset-MessageNumber*6-3+j_));
       if (what!='\r') {  buf[j__]=what; j__++; }
       else { buf[j__]=0; j__=0;  GrTextXY(x+mx,y+my+dy*y_,buf,ColorMessage,ColorBG); y_++; }
     }

/* Extract message 1 line */
     j__=0;
     for (j_=0;j_<(MessageIndex+j)->Length;j_++)
     {
       what =*(MessageData+((MessageIndex+j)->Offset-MessageNumber*6-3+j_));
       if (what!='\r') buf[j_]=what;
       else { buf[j_]=0; break; }
     }
     LocateEventOnMap(buf);
/* Show if going to map aviable */
   GrFilledBox(x+mx,y+my+dy*21,x+mx+40*FontDX,y+my+dy*22,ColorBG);
   if (MessEvent.go_to)
      GrTextXY(x+mx,y+my+dy*21,"Map",ColorMessage,ColorBG);

/* Print message number and marker */

   sprintf(buf,"%d",j+1);
   len=4;
   if (j<1000) len=3;
   if (j<100) len=2;
   if (j<10) len=1;
   GrFilledBox(x+22.0*FontDX-FontDY,y+2*dy,x+22.0*FontDX+ddx-25.0*FontDX+FontDY,y+dy*3,ColorBG);
   GrTextXY((Uns16)((float)x+22.0*FontDX+j*mdx-len*4),y+dy*2,buf,ColorMessNow,ColorBG);

//   if (j>0) GrFilledBox((Uns16)((float)x+22.0*FontDX+(j-1)*mdx),y+FontDY*1.5,(Uns16)((float)x+22.0*FontDX+(j-1)*mdx+1),y+FontDY*1.5+1,ColorMessage);
//   if (j<count-1) GrFilledBox((Uns16)((float)x+22.0*FontDX+(j+1)*mdx),y+FontDY*1.5,(Uns16)((float)x+22.0*FontDX+(j+1)*mdx+1),y+FontDY*1.5+1,ColorMessage);
   GrFilledBox((Uns16)((float)x+22.0*FontDX+lj*mdx),y+FontDY*1.5,(Uns16)((float)x+22.0*FontDX+lj*mdx+1),y+FontDY*1.5+1,ColorMessage);
   GrFilledBox((Uns16)((float)x+22.0*FontDX+j*mdx),y+FontDY*1.5,(Uns16)((float)x+22.0*FontDX+j*mdx+1),y+FontDY*1.5+1,ColorMessNow);
  } // cursor pressed
  } // key

  } // while !key_escape

}

void VCRReader(int x, int y,int ddx, int ddy)
{
  
   char buf[80];
   Uns16 lj,j_,dy=13,mx=FontDX*4,my=13*10,len;
   float mdx;
   static Uns16 j=0;

   GrMouseEvent evt;
   Uns32 c=0;

   GrFilledBox(x,y,x+ddx,y+ddy,ColorBG);

   sprintf(buf,"%3d VCR",VCRBattleNumber);
   GrTextXY(x,y+dy*1,buf,ColorSt,ColorBG);

   if (VCRBattleNumber==0) return;

   mdx = 270.0/VCRBattleNumber;
   if (mdx>10) mdx=10;
   for (j_=0;j_<VCRBattleNumber;j_++)
    GrFilledBox((Uns16)((float)x+22.0*FontDX+j_*mdx),y+FontDY*1.5,(Uns16)((float)x+22.0*FontDX+j_*mdx+1),y+FontDY*1.5+1,ColorMessage);

   lj=0;
   sprintf(buf,"%d",j+1);
   len=4;
   if (j<1000) len=3;
   if (j<100) len=2;
   if (j<10) len=1;
   GrFilledBox(x+22.0*FontDX-FontDY,y+2*dy,x+22.0*FontDX+ddx-25.0*FontDX+FontDY,y+dy*3,ColorBG);
   GrTextXY((Uns16)((float)x+22.0*FontDX+j*mdx-len*4),y+dy*2,buf,ColorMessNow,ColorBG);
   GrFilledBox((Uns16)((float)x+22.0*FontDX+j*mdx),y+FontDY*1.5,(Uns16)((float)x+22.0*FontDX+j*mdx+1),y+FontDY*1.5+1,ColorMessNow);

   GrBox(x+mx-3,y+my-3,x+mx+43*FontDX+3,y+my+dy*21+3,ColorMessage);
   GrBox(x+mx-2,y+my-2,x+mx+43*FontDX+2,y+my+dy*21+2,ColorMessage);
   ShowVCRMess(x+mx,y+my,&VCRBattle[j],VCRBattleResult+VCRBattleToBattleResult[j]);

  while (c!=GrKey_Escape)
  {
  GrMouseGetEventT(GR_M_EVENT,&evt,0L);
  if (evt.flags & (GR_M_KEYPRESS))
  {
     c=evt.key;

     if ((j<VCRBattleNumber-1)&&(c==GrKey_Right)) { lj=j; j++; }
     else
     if ((j==VCRBattleNumber-1)&&(c==GrKey_Right)) { lj=j; j=0; }

     if ((j>0)      &&(c==GrKey_Left )) { lj=j; j--; }
     else
     if ((j==0)      &&(c==GrKey_Left )) { lj=j; j=VCRBattleNumber-1; }


     if (lj!=j) // cursor pressed
     {

   ShowVCRMess(x+mx,y+my,&VCRBattle[j],VCRBattleResult+VCRBattleToBattleResult[j]);

   sprintf(buf,"%d",j+1);
   len=4;
   if (j<1000) len=3;
   if (j<100) len=2;
   if (j<10) len=1;

   GrFilledBox(x+22.0*FontDX-FontDY,y+2*dy,x+22.0*FontDX+ddx-25.0*FontDX+FontDY,y+dy*3,ColorBG);
   GrTextXY((Uns16)((float)x+22.0*FontDX+j*mdx-len*4),y+dy*2,buf,ColorMessNow,ColorBG);

   GrFilledBox((Uns16)((float)x+22.0*FontDX+lj*mdx),y+FontDY*1.5,(Uns16)((float)x+22.0*FontDX+lj*mdx+1),y+FontDY*1.5+1,ColorMessage);
   GrFilledBox((Uns16)((float)x+22.0*FontDX+j*mdx),y+FontDY*1.5,(Uns16)((float)x+22.0*FontDX+j*mdx+1),y+FontDY*1.5+1,ColorMessNow);
     } // cursor pressed
  }

  } // while key_scape

}

void ShowVCRMess(int x,int y, struct VCRMain *battle, struct util_battle_result *result)
{
   char buf[80];
   char buf1[80];
   Uns16 y_,dy=13;
   Boolean Found,Near;
   float dist;
   Uns16 pid,i;


   if (battle==NULL) return;

   y_=0;
   GrFilledBox(x,y,x+43*FontDX,y+dy*20,ColorBG);

   GrTextXY(x,y+dy*y_,"     <<< Visual Combat Recording >>>",ColorMessage,ColorBG);
   y_++;
   y_++;

   RaceNameAdjective( battle->LObject.Owner,buf);
   GrTextXY(x          ,y+dy*y_,buf,ColorMessage,ColorBG);
   RaceNameAdjective( battle->RObject.Owner,buf);
   GrTextXY(x+22*FontDX,y+dy*y_,buf,ColorMessage,ColorBG);
   GrTextXY(x+19*FontDX,y+dy*y_,"vs",ColorMessage,ColorBG);
   y_++;

   sprintf(buf,"ship #%d",battle->LObject.ID);
   GrTextXY(x     ,y+dy*y_,buf,ColorMessage,ColorBG);
   if (battle->BattleType==0)
   sprintf(buf,"ship #%d",battle->RObject.ID);
   else
   sprintf(buf,"planet #%d",battle->RObject.ID);
   GrTextXY(x+22*FontDX,y+dy*y_,buf,ColorMessage,ColorBG);
   y_++;

   strncpy(buf,battle->LObject.Name,20);
   buf[20]=0;
   GrTextXY(x     ,y+dy*y_,buf,ColorMessage,ColorBG);
   strncpy(buf,battle->RObject.Name,20);
   buf[20]=0;
   GrTextXY(x+22*FontDX,y+dy*y_,buf,ColorMessage,ColorBG);
   y_++;

   HullName(battle->LObject.Hull,buf1);
   strncpy(buf,buf1,20);
   buf[20]=0;
   GrTextXY(x     ,y+dy*y_,buf,ColorMessage,ColorBG);
   if (battle->BattleType==0)
   HullName(battle->RObject.Hull,buf1);
   else
   strcpy(buf1,"");
   strncpy(buf,buf1,20);
   buf[20]=0;
   GrTextXY(x+22*FontDX,y+dy*y_,buf,ColorMessage,ColorBG);
   y_++;

   sprintf(buf,"Mass   %4d kt",battle->LMass);
   GrTextXY(x     ,y+dy*y_,buf,ColorMessage,ColorBG);
   sprintf(buf,"Mass   %4d kt",battle->RMass);
   GrTextXY(x+22*FontDX,y+dy*y_,buf,ColorMessage,ColorBG);
   y_++;

   sprintf(buf,"Shield  %3d %%",battle->LShield);
   GrTextXY(x     ,y+dy*y_,buf,ColorMessage,ColorBG);
   sprintf(buf,"Shield  %3d %%",battle->RShield);
   GrTextXY(x+22*FontDX,y+dy*y_,buf,ColorMessage,ColorBG);
   y_++;

   sprintf(buf,"Damage  %3d %%",battle->LObject.Damage);
   GrTextXY(x     ,y+dy*y_,buf,ColorMessage,ColorBG);
   sprintf(buf,"Damage  %3d %%",battle->RObject.Damage);
   GrTextXY(x+22*FontDX,y+dy*y_,buf,ColorMessage,ColorBG);
   y_++;

   sprintf(buf,"Crew   %4d",battle->LObject.Crew);
   GrTextXY(x     ,y+dy*y_,buf,ColorMessage,ColorBG);
   if (battle->BattleType==0)
   sprintf(buf,"Crew   %4d",battle->RObject.Crew);
   else
   strcpy(buf,"");
   GrTextXY(x+22*FontDX,y+dy*y_,buf,ColorMessage,ColorBG);
   y_++;

   if (battle->LObject.BeamNumber!=0)
   {
    BeamName(battle->LObject.BeamType,buf);
    strncpy(buf1,buf,16);
    buf1[16]=0;
    sprintf(buf,"%2d x %s",battle->LObject.BeamNumber,buf1);
   }
   else
   strcpy(buf,"");
   GrTextXY(x     ,y+dy*y_,buf,ColorMessage,ColorBG);
   if (battle->RObject.BeamNumber!=0)
   {
    BeamName(battle->RObject.BeamType,buf);
    strncpy(buf1,buf,16);
    buf1[16]=0;
    sprintf(buf,"%2d x %s",battle->RObject.BeamNumber,buf1);
   }
   else
   strcpy(buf,"");
   GrTextXY(x+22*FontDX,y+dy*y_,buf,ColorMessage,ColorBG);
   y_++;

  /* Left Object = Ship allways */
   if (battle->LObject.NumBays==0)
   {
     if (battle->LObject.NumTorpLuch!=0)
     {
       TorpName(battle->LObject.TorpType,buf);
       strncpy(buf1,buf,16);
       buf1[16]=0;
       sprintf(buf,"%2d x %s",battle->LObject.NumTorpLuch,buf1);
     }
     else
       strcpy(buf,"");
   }
   else
     sprintf(buf,"%2d Fighter Bays",battle->LObject.NumBays);
   GrTextXY(x     ,y+dy*y_,buf,ColorMessage,ColorBG);
   if (battle->LObject.NumBays==0)
   {
     if (battle->LObject.NumTorpLuch!=0)
     {
       sprintf(buf,"%4d torpedos",battle->LObject.NumTorp);
     }
     else
       strcpy(buf,"");
   }
   else
     sprintf(buf,"%4d fighters",battle->LObject.NumTorp);
   GrTextXY(x     ,y+dy*(y_+1),buf,ColorMessage,ColorBG);

   /* Right Object = Ship or Planet */
   if ((battle->BattleType==0)||(gPconfigInfo->PlanetsHaveTubes==False))
   {
   if (battle->RObject.NumBays==0)
   {
     if (battle->RObject.NumTorpLuch!=0)
     {
       TorpName(battle->RObject.TorpType,buf);
       strncpy(buf1,buf,16);
       buf1[16]=0;
       sprintf(buf,"%2d x %s",battle->RObject.NumTorpLuch,buf1);
     }
     else
       strcpy(buf,"");
   }
   else
     sprintf(buf,"%2d Fighter Bays",battle->RObject.NumBays);
   GrTextXY(x+22*FontDX,y+dy*y_,buf,ColorMessage,ColorBG);

   if (battle->RObject.NumBays==0)
   {
     if (battle->RObject.NumTorpLuch!=0)
     {
       sprintf(buf,"%4d torpedos",battle->RObject.NumTorp);
     }
     else
       strcpy(buf,"");
   }
   else
     sprintf(buf,"%4d fighters",battle->RObject.NumTorp);
   GrTextXY(x+22*FontDX,y+dy*(y_+1),buf,ColorMessage,ColorBG);
  } // ship or planethavetubes=no
  else
  {
   if (battle->RObject.NumBays!=0)
     sprintf(buf,"%2d Fighter Bays",battle->RObject.NumBays);
   else
       strcpy(buf,"");
   GrTextXY(x+22*FontDX,y+dy*(y_+0),buf,ColorMessage,ColorBG);

   if (battle->RObject.NumBays!=0)
     sprintf(buf,"%4d fighters",battle->RObject.NumTorp);
   else
       strcpy(buf,"");
   GrTextXY(x+22*FontDX,y+dy*(y_+1),buf,ColorMessage,ColorBG);

   if (((battle->RObject.NumTorpLuch)&0x0ff)!=0)
     {
       TorpName(battle->RObject.TorpType,buf);
       strncpy(buf1,buf,16);
       buf1[16]=0;
       sprintf(buf,"%2d x %s",(battle->RObject.NumTorpLuch)&0x0ff,buf1);
       GrTextXY(x+22*FontDX,y+dy*(y_+2),buf,ColorMessage,ColorBG);
   if (((battle->RObject.NumTorpLuch)>>8)!=0)
       sprintf(buf,"%4d torpedos",(battle->RObject.NumTorpLuch)>>8);
   else
       strcpy(buf,"  No torpedos");
       GrTextXY(x+22*FontDX,y+dy*(y_+3),buf,ColorMessage,ColorBG);
    } // num of torp >0
  } // planets have tubes = YES

  y_++;
  y_++;
  y_++;
  y_++;
  y_++;

  /* Battle result */
  if (result==NULL) return;

  /* check if good data loaded*/
  Found=True;

  if (result->LeftID  !=battle->LObject.ID) Found=False;
  if (result->RightID !=battle->RObject.ID) Found=False;
  if (result->IsPlanet!=battle->BattleType) Found=False;
  if (Found)
  {
  strcpy(buf,VCROutcomeString(result->LeftResult));
  GrTextXY(x,y+dy*y_,buf,ColorMessage,ColorBG);
  strcpy(buf,VCROutcomeString(result->RightResult));
  GrTextXY(x+22*FontDX,y+dy*y_,buf,ColorMessage,ColorBG);
  y_++;
  y_++;

  sprintf(buf,"Battle coordinates : (%4d,%4d)",result->X,result->Y);
  GrTextXY(x,y+dy*y_,buf,ColorMessage,ColorBG);
  y_++;

  Found=False;
  for(i=1;i<=500;i++)
    if (IsPlanetExist(i))
    {
     dist=Distance(result->X,result->Y,PlanetLocationX(i),PlanetLocationY(i));
     if (dist==0.0)
     { pid = i; Near=False; Found=True; break; }
     if (dist<=3.0)
     { pid = i; Near=True;  Found=True; break; }
    }
 
  if (Found)
  {
    if (Near) sprintf(buf,"Near #%d %s",pid,PlanetName(pid,NULL));
    else sprintf(buf,"Planet #%d %s",pid,PlanetName(pid,NULL));
  }
  else
   strcpy(buf,"");
  buf[26]=0;
  GrTextXY(x+10*FontDX,y+dy*y_,buf,ColorMessage,ColorBG);
  } // good battle result

  return;
}

static char *VCROutcomeString(int OrderType)
{
   static char OrderString[40];

   strcpy(OrderString,"");

   if (OrderType==0)    strcpy(OrderString,"Winner");
   if (OrderType==1)    strcpy(OrderString,"Captured");
   if (OrderType==2)    strcpy(OrderString,"Destroyed");
   if (OrderType==3)    strcpy(OrderString,"Cannot fight");

   return OrderString;
}



/* ***************************************************************** */
/*              load bases                                           */
/* ***************************************************************** */

void  LoadBases(int tmprace)
{
   char name[20];
   FILE *f;
   Uns16 howmany,i,j;
   Uns16 buf[78];
   Uns16 id;
   BuildOrder_Struct Build;

   sprintf(name,"bdata%d.dat",tmprace);
   f=OpenInputFile(name,GAME_DIR_ONLY);

   fread(&howmany,sizeof(howmany),1,f);
   for(i=0;i<howmany;i++)
   {
     fread(&buf,sizeof(buf),1,f);

     id=buf[0];
     CreateBase(id);
     PutBaseOwner   (id,buf[1]);
     PutBaseDefense (id,buf[2]);
     PutBaseDamage  (id,buf[3]);
     PutBaseTech (id,ENGINE_TECH ,buf[4]);
     PutBaseTech (id,HULL_TECH ,buf[5]);
     PutBaseTech (id,BEAM_TECH ,buf[6]);
     PutBaseTech (id,TORP_TECH ,buf[7]);

     for (j=1;j<=ENGINE_NR;j++)
     PutBaseEngines(id,j,buf[7+j]);

     for (j=1;j<=RACEHULLS;j++)
     PutBaseHulls(id,j,buf[16+j]);

     for (j=1;j<=BEAM_NR;j++)
     PutBaseBeams(id,j,buf[36+j]);

     for (j=1;j<=TORP_NR;j++)
     PutBaseTubes(id,j,buf[46+j]);

     for (j=1;j<=TORP_NR;j++)
     PutBaseTorps(id,j,buf[56+j]);

     PutBaseFighters  (id,buf[67]);
     PutBaseFixingShip(id,buf[68]);
     PutBaseFixOrder  (id,buf[69]);
     PutBaseOrder     (id,buf[70]);

     Build.mHull      = buf[71];
     Build.mEngineType= buf[72];
     Build.mBeamType  = buf[73];
     Build.mNumBeams  = buf[74];
     Build.mTubeType  = buf[75];
     Build.mNumTubes  = buf[76];
     PutBaseBuildOrder(id,&Build);

     LocalBases[id]=LocalBases[id]|BASES_INFO_EXIST;

   }
    if (f!=NULL) fclose(f);
}

static char *BaseOrderString(int bID)
{
   Uns16 OrderType;
   static char OrderString[40];

   strcpy(OrderString,"");

   if (!IsBaseExist(bID)) return OrderString;

   OrderType = BaseOrder(bID);

   if (OrderType==NoBaseMission)    strcpy(OrderString,"None");
   if (OrderType==Refuel)           strcpy(OrderString,"Refuel");
   if (OrderType==MaxDefense)       strcpy(OrderString,"Max Defense");
   if (OrderType==LoadTorps)        strcpy(OrderString,"Load Torps onto ships");
   if (OrderType==UnloadFreighters) strcpy(OrderString,"Unload Freighters");
   if (OrderType==RepairBase)       strcpy(OrderString,"Repair Base");
   if (OrderType==ForceSurrender)   strcpy(OrderString,"Force Surrender");

   return OrderString;
}

/*
static char *BaseFixString(int bID)
{
   Uns16 OrderType;
   static char OrderString[40];

   strcpy(OrderString,"");

   if (!IsBaseExist(bID)) return OrderString;

   OrderType = BaseFixOrder(bID);

   //  Is this good ??

   if (OrderType==0)    strcpy(OrderString,"none");
   if (OrderType==1)    strcpy(OrderString,"Fix");
   if (OrderType==2)    strcpy(OrderString,"Recycle");

   return OrderString;
}
*/

Boolean ShowBase(int x,int y,int ddx, int ddy, int id)
{

  char buf[100];
  int dx=FontDX,dy=FontDY,iy=0;
  int damage;
  GrColor C;
  Boolean YesBuild;
  BuildOrder_Struct Order;

  if (!IsBaseExist(id)) return False;

  GrSetClipBox(x, y, x+ddx, y+ddy);
  GrFilledBox (x, y, x+ddx, y+ddy,ColorBG);

  sprintf(buf,"Starbase %d",id);
  GrTextXY(x,y+dy*iy,buf,ColorBase,ColorBG);
  iy++;

  sprintf(buf,"%s",PlanetName(id,NULL));
  GrTextXY(x,y+dy*iy,buf,ColorBase,ColorBG);
  iy++;
  iy++;

  sprintf(buf,"Defense:  %3d",BaseDefense(id));
  GrTextXY(x,y+dy*iy,buf,ColorBaseStandard,ColorBG);
  iy++;

  sprintf(buf,"Damage:");
  GrTextXY(x,y+dy*iy,buf,ColorBaseStandard,ColorBG);
  damage=BaseDamage(id);
  if (damage!=0)
   sprintf(buf,"%2d %%",damage);
  else
   strcpy(buf,"");
  GrTextXY(x+dx*8,y+dy*iy,buf,ColorBaseDanger,ColorBG);

  // projecte base damage when base reparing
  if ((BaseOrder(id)==RepairBase)&&(damage!=0))
  {
    if (damage-5 <= 0)
    {
      C= ColorBase;
      strcpy(buf,"(fixed)");
    }
    else
   {
     C = ColorBaseWarning;
     sprintf(buf,"(%2d%%)",damage-5);
   }
   GrTextXY(x+dx*13,y+dy*iy,buf,C,ColorBG);
  }

  iy++;

  sprintf(buf,"Fighters: %3d",BaseFighters(id));
  GrTextXY(x,y+dy*iy,buf,ColorBaseStandard,ColorBG);
  iy++;

  GrTextXY(x,y+dy*iy,"Primary order:",ColorBaseStandard,ColorBG);
  iy++;
  GrTextXY(x+dx,y+dy*iy,BaseOrderString(id),ColorBaseStandard,ColorBG);
  iy++;
  iy++;

  GrTextXY(x,y+dy*iy,"Tech. levels:",ColorBaseStandard,ColorBG);
  iy++;
  sprintf(buf,"Engines:    %2d",BaseTech(id,ENGINE_TECH));
  GrTextXY(x+dx,y+dy*iy,buf,ColorBaseStandard,ColorBG);
  iy++;
  sprintf(buf,"Hulls:      %2d",BaseTech(id,HULL_TECH));
  GrTextXY(x+dx,y+dy*iy,buf,ColorBaseStandard,ColorBG);
  iy++;
  sprintf(buf,"Weapons:    %2d",BaseTech(id,BEAM_TECH));
  GrTextXY(x+dx,y+dy*iy,buf,ColorBaseStandard,ColorBG);
  iy++;
  sprintf(buf,"Torpedoes:  %2d",BaseTech(id,TORP_TECH));
  GrTextXY(x+dx,y+dy*iy,buf,ColorBaseStandard,ColorBG);
  iy++;
  iy++;

  GrTextXY(x,y+dy*iy,"Base storage:",ColorBaseStandard,ColorBG);
  iy++;

  ShowBaseStorage(RightPanelX,RightPanelY+FontDY*16,RightPanelDX,FontDY*16,id,1);

  YesBuild=BaseBuildOrder(id,&Order);

  iy+=16;
  if (YesBuild) GrTextXY(x,y+dy*iy,"BUILDING SHIP",ColorBaseWarning,ColorBG);
  iy++;
  iy++;

  if (BaseFixOrder(id)!=0)
  {
     if (BaseFixOrder(id)==1) sprintf(buf,"FIXING Ship %-3d",BaseFixingShip(id) );
     if (BaseFixOrder(id)==2) sprintf(buf,"RECYCLING   %-3d",BaseFixingShip(id) );
     GrTextXY(x,y+dy*iy,buf,ColorBaseWarning,ColorBG);
  }

  GrResetClipBox();
  return True;
}

Uns16 ShowBaseStorage(int x,int y,int ddx, int ddy, int id, int page)
{
 /*
    This function returns shown page.
    'int page' in arguments is a desire page to show.
 */

 Uns16 i,pn,j,lines_needed=0;
 Boolean hf,ef,bf,tf,YesBuild;
 int dx=FontDX,dy=FontDY;

 char store[150][31];
 char store_color[150];
 Uns16 store_index=0;
 GrColor C;

 char buf[100];
 char buf1[100];

 BuildOrder_Struct Order;

    if (!IsBaseExist(id)) return False;

    GrSetClipBox(x,y,x+ddx,y+ddy);
    GrFilledBox(x,y,x+ddx,y+ddy,ColorBG);


    GrTextXY(x,y      ,"------------------------------",ColorBaseStandard,ColorBG);
    GrTextXY(x,y+dy*15,"------------------------------",ColorBaseStandard,ColorBG);


    YesBuild=BaseBuildOrder(id,&Order);

 // compute needed lines to show all storage
     hf=False;
     for (j=1;j<=RACEHULLS;j++)
     if (BaseHulls(id,j)>0)
     {
       lines_needed++;
       hf=True;
       for (i=0;i<BaseHulls(id,j);i++)
       {
       HullName(TrueHull(BaseOwner(id),j),store[store_index]);

       if ((i==0)&&(YesBuild)&&(Order.mHull==j))
       store_color[store_index]=1;
       else
       store_color[store_index]=0;

       store_index++;
       }
      }
     if (hf)
     {
       lines_needed++;
       strcpy(store[store_index],"------------------------------");
       store_color[store_index]=0;
       store_index++;
     }

     ef=False;
     for (j=1;j<=ENGINE_NR;j++)
     if (BaseEngines(id,j)>0) {
       lines_needed++;
       ef=True;
       EngineName(j,buf1);
       sprintf(buf,"%d %s",BaseEngines(id,j),buf1);
       strcpy(store[store_index],buf);
       if ((YesBuild)&&(Order.mEngineType==j))
       store_color[store_index]=1;
       else
       store_color[store_index]=0;
       store_index++;
       }
     if (ef)
     {
       lines_needed++;
       strcpy(store[store_index],"------------------------------");
       store_color[store_index]=0;
       store_index++;
     }

     bf=False;
     for (j=1;j<=BEAM_NR;j++)
     if (BaseBeams(id,j)>0) {
       lines_needed++;
       bf=True;
       BeamName(j,buf1);
       sprintf(buf,"%d %s",BaseBeams(id,j),buf1);
       strcpy(store[store_index],buf);
       if ((YesBuild)&&(Order.mBeamType==j))
       store_color[store_index]=1;
       else
       store_color[store_index]=0;
       store_index++;
       }

     if (bf)
     {
       lines_needed++;
       strcpy(store[store_index],"------------------------------");
       store_color[store_index]=0;
       store_index++;
     }

     tf=False;
     for (j=1;j<=TORP_NR;j++)
     if (BaseTubes(id,j)>0)
     {
       lines_needed++;
       tf=True;
       TorpName(j,buf);
       strncpy(buf1,buf,6);
       buf1[6]=0;
       sprintf(buf,"%d %s lunchers",BaseTubes(id,j),buf1);
       strcpy(store[store_index],buf);
       if ((YesBuild)&&(Order.mTubeType==j))
       store_color[store_index]=1;
       else
       store_color[store_index]=0;
       store_index++;
     }
     if (tf)
     {
       lines_needed++;
       strcpy(store[store_index],"------------------------------");
       store_color[store_index]=0;
       store_index++;
     }

     for (j=1;j<=TORP_NR;j++)
     if (BaseTorps(id,j)>0)
     {
       lines_needed++;
       TorpName(j,buf);
       strncpy(buf1,buf,6);
       buf1[6]=0;
       sprintf(buf,"%d %s torpedos",BaseTorps(id,j),buf1);
       strcpy(store[store_index],buf);
       store_color[store_index]=0;
       store_index++;
     }

   /* Display */

     pn = (lines_needed-1)/14+1;
     if (page<1) page=1;
     if (page>pn) page=pn;

     for (j=0;j<14;j++)
     {
       i=(page-1)*14+j;
       if (i<store_index)
       {
         if (store_color[i]==0) C=ColorBaseStandard;
         else                   C=ColorBaseWarning;
         GrTextXY(x,y+dy*(j+1),store[i],C,ColorBG);
       }
     }

     if (pn>1)
     {
       if (page>1)  GrTextXY(x      ,y+dy*15,"Home",ColorBase,ColorBG);
       if (page<pn) GrTextXY(x+dx*17,y+dy*15,"End" ,ColorBase,ColorBG);
     }

     GrResetClipBox();
     return page;
}

void ShowHullInfo(int x,int y,int ddx, int ddy, int id,  GrColor C)
{
     int dy=FontDY,iy=0,hull;
     char buf[100];
     

     hull=ShipHull(id);

     GrSetClipBox(x,y,x+ddx,y+ddy);
     GrFilledBox(x,y,x+ddx,y+ddy,ColorBG);

     sprintf(buf,"Hull mass : %4d kt",HullMass(hull));
     GrTextXY(x,y+dy*iy,buf,C,ColorBG);
     iy++;

     sprintf(buf,"Engines   : %4d",HullEngineNumber(hull));
     GrTextXY(x,y+dy*iy,buf,C,ColorBG);
     iy++;

     if (HullBeamNumber(hull)!=0)
       sprintf(buf,"Max Beams : %4d",HullBeamNumber(hull));
     else
       strcpy(buf,"");
     GrTextXY(x,y+dy*iy,buf,C,ColorBG);
     iy++;

     if (HullTubeNumber(hull)!=0)
       sprintf(buf,"Max Tubes : %4d",HullTubeNumber(hull));
     else
     if (HullBays(hull)!=0)
          sprintf(buf,"Hull Bays : %4d",HullBays(hull));
        else
          strcpy(buf,"");
     GrTextXY(x,y+dy*iy,buf,C,ColorBG);
     iy++;

     sprintf(buf,"Max Crew  : %4d",HullCrewComplement(hull));
     GrTextXY(x,y+dy*iy,buf,C,ColorBG);
     iy++;

     sprintf(buf,"Max Cargo : %4d",HullCargoCapacity(hull));
     GrTextXY(x,y+dy*iy,buf,C,ColorBG);
     iy++;

     sprintf(buf,"Max Fuel  : %4d",HullFuelCapacity(hull));
     GrTextXY(x,y+dy*iy,buf,C,ColorBG);
     iy++;
     iy++;

     /* Special functions, should be made in better way */

     if ( ShipDoesAlchemy(id))          {GrTextXY(x,y+dy*iy,"DoesAlchemy",C,ColorBG); iy++; }
     if ( ShipDoesRefinery(id))         {GrTextXY(x,y+dy*iy,"DoesRefinery",C,ColorBG); iy++; }
     if ( ShipHeatsTo50(id))            {GrTextXY(x,y+dy*iy,"HeatsTo50",C,ColorBG); iy++; }
     if ( ShipCoolsTo50(id))            {GrTextXY(x,y+dy*iy,"CoolsTo50",C,ColorBG); iy++; }
     if ( ShipHeatsTo100(id))           {GrTextXY(x,y+dy*iy,"HeatsTo100",C,ColorBG); iy++; }
     if ( ShipCanHyperwarp(id))         {GrTextXY(x,y+dy*iy,"CanHyperwarp",C,ColorBG); iy++; }
     if ( ShipIsGravitonic(id))         {GrTextXY(x,y+dy*iy,"IsGravitonic",C,ColorBG); iy++; }
     if ( ShipScansAllWormholes(id))    {GrTextXY(x,y+dy*iy,"ScansAllWormholes",C,ColorBG); iy++; }
     if ( ShipCanCloak(id))             {GrTextXY(x,y+dy*iy,"CanCloak",C,ColorBG); iy++; }
     if ( ShipDoesAdvancedRefinery(id)) {GrTextXY(x,y+dy*iy,"DoesAdvancedRefinery",C,ColorBG); iy++; }
     if ( ShipIsGamblingShip(id))       {GrTextXY(x,y+dy*iy,"IsGamblingShip",C,ColorBG); iy++; }
     if ( ShipIsAntiCloaking(id))       {GrTextXY(x,y+dy*iy,"IsAntiCloaking",C,ColorBG); iy++; }
     if ( ShipDoesImperialAssault(id))  {GrTextXY(x,y+dy*iy,"DoesImperialAssault",C,ColorBG); iy++; }
     if ( ShipDoesChunneling(id))       {GrTextXY(x,y+dy*iy,"DoesChunneling",C,ColorBG); iy++; }
     if ( ShipHasRamScoop(id))          {GrTextXY(x,y+dy*iy,"HasRamScoop",C,ColorBG); iy++; }
     if ( ShipDoesBioscan(id))          {GrTextXY(x,y+dy*iy,"DoesBioscan",C,ColorBG); iy++; }
     if ( ShipDoesFullBioscan(id))      {GrTextXY(x,y+dy*iy,"DoesFullBioscan",C,ColorBG); iy++; }
     if ( ShipHasAdvancedCloak(id))     {GrTextXY(x,y+dy*iy,"HasAdvancedCloak",C,ColorBG); iy++; }
     if ( ShipHasGloryDevice(id))       {GrTextXY(x,y+dy*iy,"HasGloryDevice",C,ColorBG); iy++; }
     if ( ShipHasHEGloryDevice(id))     {GrTextXY(x,y+dy*iy,"HasHEGloryDevice",C,ColorBG); iy++; }

     GrResetClipBox();
     return;
}

Boolean  LocateEventOnMap(char *buf)
{
  int ID;
  char tmp[80],*t1, *t2,c;

  t1=strchr(buf,'(');
  t2=strchr(buf,')');

  MessEvent.go_to=False;
/* '(' or  ')'not found  */
  if ((t1==NULL)||(t2==NULL)) return False;

/* '-' not found */
  if (*(t1+1)!='-') return False;

/* Try if not 'h'-host or 'a'-add on message */
  if (*(t1+2)=='a') return False;
  if (*(t1+2)=='A') return False;
  if (*(t1+2)=='h') return False;

/* get ID */
  strncpy(tmp,t1+4,3);
  tmp[3]=0;

  if (sscanf(tmp,"%d",&ID)==0) return False;

  if (ID==0) return False;

  c=tolower(*(t1+2));
  /* Explosions */
  if (c=='x')
  {
      MessEvent.x   = explosion[ID].X;
      MessEvent.y   = explosion[ID].Y;
      MessEvent.what= OBJECT_MARKER;
  }
  /* Planet */
  if ((c=='9')||(c=='d')||(c=='p')||(c=='t')||(c=='y')||(c=='z'))
  {
    if (IsPlanetExist(ID))
    {
      MessEvent.x   = PlanetLocationX(ID);
      MessEvent.y   = PlanetLocationY(ID);
      MessEvent.what= OBJECT_PLANET;
    }
    else
    return False;
  }

  /* Ship */
  if ((c=='e')||(c=='s')||(c=='w'))
  {
    if (IsShipExist(ID))
    {
    MessEvent.x=ShipLocationX(ID);
    MessEvent.y=ShipLocationY(ID);
    MessEvent.what= OBJECT_SHIP;
    }
    else
    return False;
  }

  /* Minefield */
  if ((c=='l')||(c=='m'))
  {
    if (IsMinefieldExist(ID))
    {
    MessEvent.x=MinefieldPositionX(ID);
    MessEvent.y=MinefieldPositionY(ID);
    MessEvent.what=OBJECT_MINEFIELD;
    }
    else
    return False;
  }

  /* ship or planet */
  if ((c=='f')||(c=='n'))
  {
    /* Try ship , then planet */
    if (IsShipExist(ID))
    {
    MessEvent.x=ShipLocationX(ID);
    MessEvent.y=ShipLocationY(ID);
    MessEvent.what= OBJECT_SHIP;
    }
    else
    if (IsPlanetExist(ID))
    {
    MessEvent.x=PlanetLocationX(ID);
    MessEvent.y=PlanetLocationY(ID);
    MessEvent.what= OBJECT_PLANET;
    }
    else
    return False;
  }

  MessEvent.go_to=True;
  MessEvent.id   =ID;
  return True;
}

